VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "InfoComponent"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Public manifest_ As InfoManifest

Public sourceHome_ As String
Public artifactHome_ As String

Private version_ As String
Private appWrap_ As Object
Private artSource_ As API_Project
Private bReadOnly_ As Boolean

Public Function Init(sManifest$, sSources$, sArtifacts$, sVersion$) As Boolean
  Set manifest_ = New InfoManifest
  sourceHome_ = sSources
  artifactHome_ = sArtifacts
  version_ = sVersion
  Init = manifest_.Init(sManifest)
End Function

' == Properties =
Public Property Get ItemName() As String
  ItemName = manifest_.properties_(MANIFEST_PROPS_NAME)
End Property

Public Property Get Artifact() As String
  Artifact = CPath(manifest_.properties_(MANIFEST_PROPS_ARTIFACT)).ToGlobal(artifactHome_).Text
End Property

Public Property Get Skeleton() As String
  Skeleton = sourceHome_ & "\" & SUBFOLDER_SKELETON & "\" & ItemName
End Property

' ===== API =====
Public Function ValidateManifest(bType As TBuildType) As Collection ' of string
  If Not OpenArtifact(bReadOnly:=True) Then _
    Exit Function
  Set ValidateManifest = manifest_.Validate(bType, appWrap_.Document.VBProject)
  Call ReleaseArtifact(bSave:=False)
End Function

Public Function OpenFileInNewProcess(bReadOnly As Boolean) As Object
  Dim iWrap As Object: Set iWrap = AccessArtifact(Artifact, bReadOnly)
  If Not iWrap Is Nothing Then _
    Set OpenFileInNewProcess = iWrap.Document
End Function

Public Function ClearCode() As Boolean
  ClearCode = OpenArtifact(bReadOnly:=False)
  If ClearCode Then
    Call artSource_.RemoveAll
    Call ReleaseArtifact(bSave:=True)
  End If
End Function

Public Function ExportCode() As Boolean
  ExportCode = OpenArtifact(bReadOnly:=True)
  If ExportCode Then
    Call artSource_.ExportSrcTo(sourceHome_)
    Call ReleaseArtifact(bSave:=False)
    ExportCode = manifest_.ExportUI(Artifact, sourceHome_ & "\" & SUBFOLDER_UI)
  End If
End Function

Public Function ExportShared() As Boolean
  ExportShared = OpenArtifact(bReadOnly:=True)
  If ExportShared Then
    Call artSource_.ExportShared
    Call ReleaseArtifact(bSave:=False)
  End If
End Function

Public Function ExportAll() As Boolean
  ExportAll = OpenArtifact(bReadOnly:=True)
  If ExportAll Then
    Call artSource_.ExportSrcTo(sourceHome_)
    Call artSource_.ExportShared
    Call ReleaseArtifact(bSave:=False)
    ExportAll = manifest_.ExportUI(Artifact, sourceHome_ & "\" & SUBFOLDER_UI)
  End If
End Function

Public Function ReloadCode() As Boolean
  ReloadCode = False
  If Not OpenArtifact(bReadOnly:=False) Then _
    Exit Function
  Call artSource_.ImportSrcFrom(sourceHome_)
  ReloadCode = manifest_.ImportRefs(appWrap_.Document.VBProject)
  Call ReleaseArtifact(bSave:=True)
  
  If ReloadCode Then _
    ReloadCode = manifest_.ImportUI(sourceHome_ & "\" & SUBFOLDER_UI, Artifact)
End Function

Public Function ReloadShared() As Boolean
  ReloadShared = OpenArtifact(bReadOnly:=False)
  If ReloadShared Then
    Call artSource_.ImportShared
    ReloadShared = manifest_.ImportRefs(appWrap_.Document.VBProject)
    If ReloadShared Then _
      ReloadShared = CompileVBProject(appWrap_.Document.VBProject)
    Call ReleaseArtifact(bSave:=True)
  End If
End Function

Public Function ReloadAll() As Boolean
  ReloadAll = OpenArtifact(bReadOnly:=False)
  If Not ReloadAll Then _
    Exit Function
  
  Call artSource_.RemoveAll
  Call artSource_.ImportSrcFrom(sourceHome_)
  Call artSource_.ImportShared
  
  ReloadAll = manifest_.ImportRefs(appWrap_.Document.VBProject)
  Call ReleaseArtifact(bSave:=True)
  
  If ReloadAll Then _
    ReloadAll = manifest_.ImportUI(sourceHome_ & "\" & SUBFOLDER_UI, Artifact)
End Function

Public Function UpdateSkeleton() As Boolean
  UpdateSkeleton = manifest_.ExportUI(Artifact, sourceHome_ & "\" & SUBFOLDER_UI)
  If Not UpdateSkeleton Then _
    Exit Function
    
  UpdateSkeleton = OpenArtifact(bReadOnly:=False)
  If Not UpdateSkeleton Then _
    Exit Function
    
  Call artSource_.ExportSrcTo(sourceHome_)
  Call artSource_.ExportShared
    
  Call PrepareSkeletonFor(appWrap_.Document)
  Call artSource_.RemoveAll
  
  Dim fso As New Scripting.FileSystemObject
  Dim sSkeletonFile$: sSkeletonFile = Skeleton
  Call EnsureFolderExists(fso.GetParentFolderName(sSkeletonFile), fso)
  Call appWrap_.SaveAs(sSkeletonFile)
  
  Call ReleaseArtifact(bSave:=True)
End Function

Public Function Clear()
  Dim fso As New Scripting.FileSystemObject
  Dim sFile$: sFile = Artifact
  If fso.FileExists(sFile) Then _
    Call fso.DeleteFile(sFile)
End Function

Public Function Compile() As Boolean
  Compile = OpenArtifact(bReadOnly:=True)
  If Compile Then
    Compile = CompileVBProject(appWrap_.Document.VBProject)
    Call ReleaseArtifact(bSave:=False)
  End If
End Function

Public Function Test(ByRef reporter As API_Logger, ByRef iTestOut As IteratorTest) As CDS_InfoTests
  If Not OpenArtifact(bReadOnly:=True) Then _
    Exit Function
  Set Test = RunTestsInternal(reporter, iTestOut)
  Call ReleaseArtifact(bSave:=False)
End Function

Public Function Build(bType As TBuildType, ByRef reporter As API_Logger, ByRef iTestOut As IteratorTest) As InfoBuild
  Set Build = New InfoBuild
  Build.status_ = T_BS_FAILED
  Call reporter.Log("Start building component: " & ItemName)
  Call reporter.Log("Build type: " & BuildDescription(bType))
  
  Dim sArtifact$: sArtifact = Artifact
  Call reporter.Log("Ensure artifact exists: " & sArtifact)
  If Not EnsureArtifactExists() Then _
    Exit Function
  
  Call reporter.Log("Accessing artifact file: " & sArtifact)
  If Not OpenArtifact(bReadOnly:=False) Then _
    Exit Function
    
  Call reporter.Log("Loading code modules...")
  Call artSource_.ImportSrcFrom(sourceHome_)
  Call artSource_.ImportShared
  Call manifest_.ImportRefs(appWrap_.Document.VBProject)
  
  If version_ <> vbNullString Then
    Call reporter.Log("Writing version number...")
    Call UpdateVersionStamp(appWrap_.Document.VBProject, version_)
  End If
  
  Call reporter.Log("Compiling VBProject...")
  If Not CompileVBProject(appWrap_.Document.VBProject) Then
    Call reporter.Log("Compilation failed")
    GoTo SAFE_EXIT
  End If
  
  Call ReleaseArtifact(bSave:=True)
  
  Call reporter.Log("Importing UI elements...")
  If Not manifest_.ImportUI(sourceHome_ & "\" & SUBFOLDER_UI, sArtifact) Then _
    Exit Function
    
  If Not OpenArtifact(bReadOnly:=True) Then _
    Exit Function
    
  Call reporter.Log("Validating manifest...")
  Dim oErrors As Collection: Set oErrors = manifest_.Validate(bType, appWrap_.Document.VBProject)
  If oErrors Is Nothing Then
    Call reporter.Log("Validation failed")
    GoTo SAFE_EXIT
  ElseIf oErrors.Count > 0 Then
    Call reporter.Log("Validation failed")
    GoTo SAFE_EXIT
  End If
    
  Call reporter.Log("Testing VBProject...")
  Dim testResults As CDS_InfoTests: Set testResults = RunTestsInternal(reporter, iTestOut)
  Set Build.tests_ = testResults
  If testResults Is Nothing Then _
    GoTo SAFE_EXIT
  Call reporter.Log("Tests summary: " & testResults.count_ & " | " & testResults.success_ & " / " & testResults.failed_)
  If testResults.failed_ <> 0 Then
    Call reporter.Log("Tests failed... interrupting build")
    GoTo SAFE_EXIT
  End If

  If testResults.failed_ <> 0 Then _
    GoTo SAFE_EXIT
    
  If bType = T_BUILD_RELEASE Then
    Call ReleaseArtifact(bSave:=False)
    If Not OpenArtifact(bReadOnly:=False) Then _
      Exit Function
    Call reporter.Log("Removing debug code for release build...")
    Call Dev_RemoveDebugCode(appWrap_.Document.VBProject)
    If Not CompileVBProject(appWrap_.Document.VBProject) Then
      Call reporter.Log("Compilation failed")
      GoTo SAFE_EXIT
    End If
  End If

  Build.status_ = T_BS_OK
SAFE_EXIT:
  Call ReleaseArtifact(bSave:=True)
  Call reporter.Log("Finish building component: " & ItemName)
End Function

' ====
Private Function EnsureArtifactExists() As Boolean
  EnsureArtifactExists = False
  
  Dim sArtifact$: sArtifact = Artifact
  Dim fso As New FileSystemObject
  If Not fso.FileExists(sArtifact) Then
    Dim sSkeleton$: sSkeleton = Skeleton
    If Not fso.FileExists(sSkeleton) Then _
      Exit Function
    Call CopyFileOrFolder(sSkeleton, sArtifact, fso)
  End If
  
  EnsureArtifactExists = True
End Function

Private Function OpenArtifact(bReadOnly As Boolean) As Boolean
  Set appWrap_ = AccessArtifact(Artifact, bReadOnly)
  If appWrap_ Is Nothing Then
    OpenArtifact = False
    Exit Function
  End If
  
  Set artSource_ = New API_Project
  OpenArtifact = artSource_.Init(appWrap_.Document.VBProject, Globals.SharedHome)
  If OpenArtifact Then
    bReadOnly_ = bReadOnly
    Call manifest_.InitSourceAPI(artSource_)
  Else
    Call ReleaseArtifact(bSave:=False)
  End If
End Function

Private Function ReleaseArtifact(bSave As Boolean)
  If Not artSource_ Is Nothing Then
    Call artSource_.Detach
    Set artSource_ = Nothing
  End If
  If Not appWrap_ Is Nothing Then
    Call appWrap_.ReleaseDocument(bSaveChanges:=Not bReadOnly_ And bSave)
    Set appWrap_ = Nothing
  End If
End Function

Private Function RunTestsInternal(ByRef reporter As API_Logger, ByRef iTestOut As IteratorTest) As CDS_InfoTests
  Dim runner As New API_TestRunner
  Call runner.Init(appWrap_, reporter, iTestOut)
  Set RunTestsInternal = runner.RunAllTests()
  Set runner = Nothing
End Function
