VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "InfoMakefile"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Public sPath_ As String

Public properties_ As Scripting.Dictionary
Public buildActs_ As Collection ' of InfoAction
Public installActs_ As Collection ' of InfoAction

Public Function Init(sFile$) As Boolean
  sPath_ = sFile
  
  Set properties_ = New Scripting.Dictionary
  Set buildActs_ = New Collection
  Set installActs_ = New Collection
  
  Init = LoadFromFile
End Function

Public Property Get ProductID() As String
  ProductID = properties_(MAKEFILE_PROPS_ID)
End Property

Public Property Get ProductName() As String
  ProductName = properties_(MAKEFILE_PROPS_NAME)
End Property

Public Function Validate(sSourceHome$, sArtifactHome$, bIsBuilding As Boolean) As Collection
  On Error GoTo RETURN_NOTHING
  
  Dim oLog As New Collection
  Dim sArtifacts As New Scripting.Dictionary
  Set sArtifacts = ValidateBuildActions(sSourceHome, oLog)
  Call ValidateInstallActions(sArtifactHome, oLog, sArtifacts, bIsBuilding)
  
  Set Validate = oLog
RETURN_NOTHING:
End Function

' =======
Private Function LoadFromFile() As Boolean
  LoadFromFile = False
  
  Dim fso As New Scripting.FileSystemObject
  If Not fso.FileExists(sPath_) Then _
    Exit Function
  
  Dim adoStream As New ADODB.Stream
  adoStream.Charset = "utf-8"
  Call adoStream.Open
  Call adoStream.LoadFromFile(sPath_)
  Dim sLines() As String: sLines = Split(adoStream.ReadText, vbCrLf)
  Call adoStream.Close
  
  Dim nCurrent&: nCurrent = LBound(sLines, 1)
  Dim nLast&: nLast = UBound(sLines, 1)
  If Not ScanProperties(sLines, nCurrent, nLast) Then _
    Exit Function
  If Not ScanActions(sLines, nCurrent, nLast, buildActs_) Then _
    Exit Function
  If Not ScanActions(sLines, nCurrent, nLast, installActs_) Then _
    Exit Function

  LoadFromFile = True
End Function

Private Function ScanProperties(sLines() As String, ByRef nCurrent&, nLast&) As Boolean
  ScanProperties = ExtractProperties(sLines, nCurrent, properties_)
  If Not ScanProperties Then _
    Exit Function
  ScanProperties = _
    properties_.Exists(MAKEFILE_PROPS_ID) _
    And properties_.Exists(MAKEFILE_PROPS_NAME) _
    And properties_.Exists(MAKEFILE_PROPS_ARTIFACT) _
    And properties_.Exists(MAKEFILE_PROPS_SOURCE) _
    And properties_.Exists(MAKEFILE_PROPS_INSTALL) _
    And nCurrent <= nLast
End Function

Private Function ScanActions(sLines() As String, ByRef nCurrent&, nLast&, ByRef oSink As Collection) As Boolean
  ScanActions = False
  
  Dim iAction As InfoAction
  Do While nCurrent <= nLast
    Dim sLine$: sLine = sLines(nCurrent)
    nCurrent = nCurrent + 1
    If sLine Like "%%*" Then _
      Exit Do
    If Left(sLine, 1) = "#" Then _
      GoTo NEXT_LINE
    
    
    Dim nSpace&: nSpace = InStr(1, sLine, " ")
    If nSpace = 0 Then _
      GoTo NEXT_LINE
      
    Set iAction = New InfoAction
    
    iAction.type_ = Text2ActionType(Trim(Left(sLine, nSpace)))
    sLine = Trim(Right(sLine, Len(sLine) - nSpace))
    
    Dim nSeparator&: nSeparator = InStr(1, sLine, IIf(iAction.type_ = T_ACT_RUN, "<-", "->"))
    If nSeparator = 0 Then
      Call iAction.args_.Add(Trim(sLine))
    Else
      Call iAction.args_.Add(Trim(Left(sLine, nSeparator - 1)))
      Call iAction.args_.Add(Trim(Right(sLine, Len(sLine) - nSeparator - 2)))
    End If
    Call oSink.Add(iAction)
    
NEXT_LINE:
  Loop
  
  ScanActions = True
End Function

Private Function ValidateBuildActions(sSourceHome$, oLog As Collection) As Scripting.Dictionary
  Dim sArtifacts As New Scripting.Dictionary
  
  Dim anAction As InfoAction
  For Each anAction In buildActs_
    Dim arg1$: arg1 = anAction.args_(1)
    Dim iPath As API_Path: Set iPath = CPath(arg1).ToGlobal(sSourceHome)
    If anAction.type_ <> T_ACT_SAVE_AS Then
      If anAction.type_ <> T_ACT_RUN Then _
        If Not iPath.GlobalExists() Then _
          Call oLog.Add("Missing source: " & arg1)
    Else
      If Not sArtifacts.Exists(arg1) Then _
        Call oLog.Add("Unknown artifact: " & arg1)
    End If
    
    If anAction.IsInstallOnly Or anAction.type_ = T_ACT_UNDEF Then _
      Call oLog.Add("Invalid build action type: " & anAction.Description)
    
    If anAction.type_ = T_ACT_BUILD Then
      Dim iManifest As New InfoManifest
      If iManifest.Init(iPath.Text) Then
        Call sArtifacts.Add(iManifest.properties_(MANIFEST_PROPS_ARTIFACT), "")
      End If
    End If
    
    If anAction.args_.Count > 1 Then _
      Call sArtifacts.Add(anAction.args_(2), "")
    If anAction.args_.Count = 1 And anAction.type_ = T_ACT_COPY Then
      If iPath.Extension = vbNullString Then
        Call sArtifacts.Add(iPath.BaseName, "")
      Else
        Call sArtifacts.Add(iPath.FileName, "")
      End If
    End If
  Next anAction
  
  Set ValidateBuildActions = sArtifacts
End Function

Private Function ValidateInstallActions(sArtifactHome$, oLog As Collection, sArtifacts As Scripting.Dictionary, bIsBuilding As Boolean)
  Dim anAction As InfoAction
  For Each anAction In installActs_
    If anAction.IsBuildOnly Or anAction.type_ = T_ACT_UNDEF Then _
      Call oLog.Add("Invalid install action type: " & anAction.Description)
    
    Dim arg1$: arg1 = anAction.args_(1)
    Dim iPath As API_Path: Set iPath = CPath(arg1).ToGlobal(sArtifactHome)
    If bIsBuilding Then
      If Not sArtifacts.Exists(arg1) Then _
        Call oLog.Add("Unknown artifact: " & arg1)
    Else
      If Not iPath.GlobalExists Then _
        Call oLog.Add("Missing artifact: " & iPath.Text)
    End If
  Next anAction
End Function

