VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "InfoManifest"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Public sPath_ As String

Public properties_ As Scripting.Dictionary
Public contents_ As Scripting.Dictionary
Public sharedContents_ As Scripting.Dictionary
Public uiElements_ As Collection ' of DescriptorUI
Public references_ As Collection ' of ItemVBReference

Private fso_ As Scripting.FileSystemObject

Private Const IMPORTS_MAX_DEPTH = 9

Public Function Init(sManifestPath$) As Boolean
  sPath_ = sManifestPath$
  
  Set properties_ = New Scripting.Dictionary
  
  Set contents_ = New Scripting.Dictionary
  Set sharedContents_ = New Scripting.Dictionary
  
  Set uiElements_ = New Collection
  Set references_ = New Collection
  
  Set fso_ = New Scripting.FileSystemObject
  
  Init = LoadFromFile
End Function

Public Function InitSourceAPI(ByRef target As API_Project)
  Call target.SetInternals(contents_, sharedContents_)
End Function

Public Function AddSrcItem(sName$, sFolder$)
  Call contents_.Add(sName, sFolder)
End Function

Public Function AddSharedItem(sName$, sFolder$)
  Call sharedContents_.Add(sName, sFolder)
End Function

Public Function ExportUI(sSourceFile$, sDestinationFolder$) As Boolean
  ExportUI = False
  
  Dim sZip$: sZip = sDestinationFolder & "\" & "uiTMP"
  If Not UnzipFile(sSourceFile, sZip) Then _
    Exit Function
  
  Dim itemInfo As DescriptorUI
  Dim sSourceUI$
  Dim sDestUI$
  For Each itemInfo In uiElements_
    sSourceUI = sZip & "\" & itemInfo.internalPath_
    sDestUI = sDestinationFolder & "\" & itemInfo.sourcePath_
    Call EnsureFolderExists(fso_.GetParentFolderName(sDestUI))
    Call fso_.CopyFile(sSourceUI, sDestUI)
  Next itemInfo
  
  Call fso_.DeleteFolder(sZip)
  
  ExportUI = True
End Function

Public Function ImportUI(sSourceFolder$, sDestinationFile$) As Boolean
  ImportUI = False
  
  Dim sZip$: sZip = sSourceFolder & "\" & "uiTMP"
  If Not UnzipFile(sDestinationFile, sZip) Then _
    Exit Function

  Dim itemInfo As DescriptorUI
  Dim sSourceUI$
  Dim sDestUI$
  For Each itemInfo In uiElements_
    sDestUI = sZip & "\" & itemInfo.internalPath_
    sSourceUI = sSourceFolder & "\" & itemInfo.sourcePath_
    Call EnsureFolderExists(fso_.GetParentFolderName(sDestUI))
    Call fso_.CopyFile(sSourceUI, sDestUI)
  Next itemInfo
  
  ImportUI = ZipFolder(sZip, sDestinationFile)
  Call fso_.DeleteFolder(sZip)
End Function

Public Function ImportRefs(target As VBIDE.VBProject) As Boolean
  ImportRefs = False
  
  Dim iReferences As DB_GlobalRefs: Set iReferences = Globals.References
  Dim iRef As ItemVBReference
  For Each iRef In references_
    Select Case iRef.type_
      Case T_REF_GLOBAL_NAME
        Dim theRef As ItemVBReference: Set theRef = iReferences.GetGlobal(iRef.id_)
        If theRef Is Nothing Then
          Debug.Print "Invalid global name " & iRef.id_
          Exit Function
        End If
        If Not TryImportRefGUID(target, theRef.id_, theRef.major_, theRef.minor_) Then _
          Exit Function
      Case T_REF_GUID
        If Not TryImportRefGUID(target, iRef.id_) Then _
          Exit Function
      Case T_REF_FILE
        If Not TryImportRefFile(iRef.id_, target) Then _
          Exit Function
    End Select
  Next iRef
  
  ImportRefs = True
End Function

Public Function Validate(bType As TBuildType, target As VBIDE.VBProject) As Collection
  Dim oLog As New Collection
  
  Call ValidateCode(bType, target, Globals.SharedHome, oLog)
  Call ValidateRefs(target, Globals.References, oLog)
  
  Set Validate = oLog
End Function

' =======
Private Function LoadFromFile() As Boolean
  LoadFromFile = False
  
  If Not fso_.FileExists(sPath_) Then _
    Exit Function
  
  Dim adoStream As New ADODB.Stream
  adoStream.Charset = "utf-8"
  Call adoStream.Open
  Call adoStream.LoadFromFile(sPath_)
  Dim sLines() As String: sLines = Split(adoStream.ReadText, vbCrLf)
  Call adoStream.Close
  
  Dim nCurrent&: nCurrent = LBound(sLines, 1)
  Dim nLast&: nLast = UBound(sLines, 1)
  If Not ScanProperties(sLines, nCurrent, nLast) Then _
    Exit Function
  If Not ScanStructure(sLines, nCurrent, nLast, "AddSharedItem") Then _
    Exit Function
  If Not ScanStructure(sLines, nCurrent, nLast, "AddSrcItem") Then _
    Exit Function
  If Not ScanSubstitutes(sLines, nCurrent, nLast) Then _
    Exit Function
  If Not ScanReferences(sLines, nCurrent, nLast) Then _
    Exit Function

  LoadFromFile = True
End Function

Private Function ScanProperties(sLines() As String, ByRef nCurrent&, nLast&) As Boolean
  ScanProperties = ExtractProperties(sLines, nCurrent, properties_)
  If Not ScanProperties Then _
    Exit Function
  ScanProperties = properties_.Exists(MANIFEST_PROPS_NAME) _
    And properties_.Exists(MANIFEST_PROPS_ARTIFACT) _
    And nCurrent <= nLast
End Function

Private Function ScanStructure(sLines() As String, ByRef nCurrent&, nLast&, sCallback$) As Boolean
  ScanStructure = False
  
  Dim sParts(0 To IMPORTS_MAX_DEPTH) As String
  Dim lastLvl&: lastLvl = -1
  Do While nCurrent <= nLast
    Dim sLine$: sLine = sLines(nCurrent)
    nCurrent = nCurrent + 1
    If sLine Like "%%*" Then _
      Exit Do
      
    Dim sPart$: sPart = Replace(sLine, vbTab, "")
    If sPart = vbNullString Then _
      GoTo NEXT_LINE
    If Left(sPart, 1) = "#" Then _
      GoTo NEXT_LINE
    
    Dim nLevel&: nLevel = Len(sLine) - Len(sPart)
    sPart = Trim(sPart)
    If nLevel > lastLvl Then _
      GoTo NEXT_PART
      
    Dim sFile$: sFile = sParts(lastLvl)
    If Not IsVBAModuleExtension(fso_.GetExtensionName(sFile)) Then _
      GoTo NEXT_PART
    
    Call CallByName(Me, sCallback, VbMethod, sFile, ConstructPath(sParts, 0, lastLvl - 1))
    
NEXT_PART:
    sParts(nLevel) = sPart
    lastLvl = nLevel
    
NEXT_LINE:
  Loop
  If lastLvl <> -1 Then _
    If IsVBAModuleExtension(fso_.GetExtensionName(sParts(lastLvl))) Then _
      Call CallByName(Me, sCallback, VbMethod, sParts(lastLvl), ConstructPath(sParts, 0, lastLvl - 1))
  ScanStructure = nCurrent <= nLast
End Function

Private Function ScanSubstitutes(sLines() As String, ByRef nCurrent&, nLast&) As Boolean
  Dim oNewUI As DescriptorUI
  Do While nCurrent <= nLast
    Dim sLine$: sLine = sLines(nCurrent)
    nCurrent = nCurrent + 1
    If sLine Like "%%*" Then _
      Exit Do

    Dim nSeparator&: nSeparator = InStr(1, sLine, "->")
    If nSeparator = 0 Then _
      GoTo NEXT_LINE
    If Left(sLine, 1) = "#" Then _
      GoTo NEXT_LINE
    
    Set oNewUI = New DescriptorUI
    oNewUI.sourcePath_ = Trim(Left(sLine, nSeparator - 1))
    oNewUI.internalPath_ = Trim(Right(sLine, Len(sLine) - nSeparator - 1))
    Call uiElements_.Add(oNewUI)
    
NEXT_LINE:
  Loop
  ScanSubstitutes = nCurrent <= nLast
End Function

Private Function ScanReferences(sLines() As String, ByRef nCurrent&, nLast&) As Boolean
  ScanReferences = False
  
  Dim oRef As ItemVBReference
  Do While nCurrent <= nLast
    Dim sLine$: sLine = sLines(nCurrent)
    nCurrent = nCurrent + 1
    
    Dim nSeparator&: nSeparator = InStr(1, sLine, ":")
    If nSeparator = 0 Then _
      GoTo NEXT_LINE
    If Left(sLine, 1) = "#" Then _
      GoTo NEXT_LINE
    
    Set oRef = New ItemVBReference
    oRef.type_ = RefTypeFrom(Trim(Left(sLine, nSeparator - 1)))
    If oRef.type_ = T_REF_UNDEF Then _
      Exit Function
    oRef.id_ = Trim(Right(sLine, Len(sLine) - nSeparator))
    
    Call references_.Add(oRef)

NEXT_LINE:
  Loop
  
  ScanReferences = True
End Function

Private Function ConstructPath(sParts() As String, nFirst&, nLast&) As String
  Dim nItem&
  For nItem = nFirst To nLast Step 1
    ConstructPath = ConstructPath & sParts(nItem) & "\"
  Next nItem
  ConstructPath = Left(ConstructPath, Len(ConstructPath) - 1)
End Function

Private Function ComponentName(target As VBComponent) As String
  ComponentName = target.Name
  Select Case target.Type
    Case vbext_ct_ClassModule: ComponentName = ComponentName & ".cls"
    Case vbext_ct_MSForm: ComponentName = ComponentName & ".frm"
    Case vbext_ct_StdModule: ComponentName = ComponentName & ".bas"
    Case Else: ComponentName = vbNullString
  End Select
End Function

Private Function ValidateCode(bType As TBuildType, target As VBIDE.VBProject, sSharedHome$, ByRef oLog As Collection)
  Dim oNames As New Collection
  Dim aComp As VBIDE.VBComponent
  For Each aComp In target.VBComponents
    If aComp.Type = vbext_ct_Document Then _
      GoTo NEXT_COMP
    
    Dim sName$: sName = ComponentName(aComp)
    Call oNames.Add("", sName)
    If contents_.Exists(sName) Then _
      GoTo NEXT_COMP
    If sharedContents_.Exists(sName) Then _
      GoTo NEXT_COMP
    
    Call oLog.Add("Unlisted module: " & sName)
    
NEXT_COMP:
  Next aComp

  Dim aName As Variant
  For Each aName In contents_
    If bType = T_BUILD_RELEASE Then _
      If Dev_IsTestingModule(CStr(aName)) Then _
        GoTo NEXT_NAME
    If Not InCollection(CStr(aName), oNames) Then _
      Call oLog.Add("Missing module: " & aName)
NEXT_NAME:
  Next aName
  
  For Each aName In sharedContents_
    If Not InCollection(CStr(aName), oNames) Then _
      Call oLog.Add("Missing shared module: " & aName)
    Dim sItemPath$: sItemPath = sSharedHome & "\" & sharedContents_(aName) & "\" & aName
    If Not fso_.FileExists(sItemPath) Then _
      Call oLog.Add("Missing shared file: " & sItemPath)
  Next aName
End Function

Private Function ValidateRefs(target As VBIDE.VBProject, Globals As DB_GlobalRefs, ByRef oLog As Collection)
  Dim projRef As VBIDE.Reference
  For Each projRef In target.References
    If Not ValidateProjectRef(projRef, target, Globals) Then _
      Call oLog.Add("Unlisted reference: " & projRef.Name)
  Next projRef

  Dim iRef As ItemVBReference
  For Each iRef In references_
    If Not ValidateManifestRef(target, iRef, Globals) Then _
      Call oLog.Add("Missing reference: " & iRef.ToString)
  Next iRef
End Function

Private Function ValidateProjectRef(target As VBIDE.Reference, proj As VBIDE.VBProject, Globals As DB_GlobalRefs) As Boolean
  If target.Type = vbext_rk_Project Then
    ValidateProjectRef = True
    Exit Function
  End If
  If IsBasicReference(target, proj) Then
    ValidateProjectRef = True
    Exit Function
  End If
  Dim iRef As ItemVBReference
  For Each iRef In references_
    If iRef.Test(target) Then
      ValidateProjectRef = True
      Exit Function
    End If
  Next iRef
  ValidateProjectRef = False
End Function

Private Function ValidateManifestRef(proj As VBIDE.VBProject, target As ItemVBReference, Globals As DB_GlobalRefs) As Boolean
  Dim aRef As VBIDE.Reference
  For Each aRef In proj.References
    If target.Test(aRef) Then
      ValidateManifestRef = True
      Exit Function
    End If
  Next aRef
  ValidateManifestRef = False
End Function

Private Function TryImportRefGUID(target As VBIDE.VBProject, sID$, Optional nMajor& = 0, Optional nMinor& = 0) As Boolean
  Dim aRef As VBIDE.Reference
  For Each aRef In target.References
    If aRef.GUID = sID Then
      TryImportRefGUID = True
      Exit Function
    End If
  Next aRef
  
  On Error GoTo PARSE_ERRORS
  Dim newRef As VBIDE.Reference: Set newRef = target.References.AddFromGuid(sID, nMajor, nMinor)
  On Error GoTo 0
  
  TryImportRefGUID = Not newRef.IsBroken
  
  Exit Function
PARSE_ERRORS:
  TryImportRefGUID = ProcessImportError(Err.Number, Err.Description, sID)
End Function

Private Function TryImportRefFile(sFile$, target As VBIDE.VBProject) As Boolean
  Dim aRef As VBIDE.Reference
  For Each aRef In target.References
    If aRef.FullPath = sFile Then
      TryImportRefFile = True
      Exit Function
    End If
  Next aRef
  
  On Error GoTo PARSE_ERRORS
  Dim newRef As VBIDE.Reference: Set newRef = target.References.AddFromFile(sFile)
  On Error GoTo 0
  
  TryImportRefFile = newRef.IsBroken
  
  Exit Function
PARSE_ERRORS:
  TryImportRefFile = ProcessImportError(Err.Number, Err.Description, sFile)
End Function

Private Function ProcessImportError(nID&, sDescription$, sData$) As Boolean
  ProcessImportError = False
  Select Case Err.Number
    Case 48: Debug.Print "Failed loading " & sData
    Case 1004: Debug.Print "Trust center is not cofigured properly"
    Case -2147319779: Debug.Print "Unknown GUID " & sData
    Case 32813
      Debug.Print "Already loaded " & sData
      ProcessImportError = True
    Case Else: Debug.Print "Unknown error " & nID & " " & sDescription
  End Select
End Function
