VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "InfoProduct"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Public make_ As InfoMakefile
Public version_ As String
Public distribute_ As Boolean

Public Function Init(sMakefile$, sVersion$, doDistribute As Boolean) As Boolean
  Set make_ = New InfoMakefile
  Init = make_.Init(sMakefile)
  distribute_ = doDistribute
  version_ = sVersion
End Function

' == Properties =
Public Property Get ProductID() As String
  ProductID = make_.properties_(MAKEFILE_PROPS_ID)
End Property

Public Property Get Name() As String
  Name = make_.properties_(MAKEFILE_PROPS_NAME)
End Property

Public Property Get SourceHome() As String
   SourceHome = CPath(make_.properties_(MAKEFILE_PROPS_SOURCE)).ToGlobal(Globals.SourceHome).Text
End Property

Public Property Get ArtifactHome() As String
   ArtifactHome = CPath(make_.properties_(MAKEFILE_PROPS_ARTIFACT)).ToGlobal(Globals.ArtifactHome).Text
End Property

Public Property Get InstallHome() As String
   InstallHome = make_.properties_(MAKEFILE_PROPS_INSTALL)
End Property

' ===== Actions ======
Public Function ValidateMakefile() As Collection ' of string
  Set ValidateMakefile = make_.Validate(SourceHome, ArtifactHome, bIsBuilding:=True)
End Function

Public Function Build(bType As TBuildType, reporter As API_Logger, dbComponents As DB_Components, testDB As DB_Tests, Optional sArtPath$ = vbNullString) As Boolean
  Build = False
  
  If sArtPath = vbNullString Then _
    sArtPath = ArtifactHome
  
  Dim env As New ItemActionEnvironment
  Call env.InitBuild(bType, Name, version_, sArtPath, SourceHome, reporter, dbComponents, testDB)
  
  Call reporter.Log("Start building product: " & Name)
  Call reporter.Log("Version: " & version_)
  
  If version_ <> vbNullString Then
    Call reporter.Log("Updating version file: " & version_)
    If Not VersionCreateFile(env.sourceHome_, version_) Then _
      Exit Function
  End If
  
  Call reporter.Log("Validating makefile...")
  If Not ActValidateMakefile(env) Then _
    Exit Function
  
  Call reporter.Log("Processing build actions...")
  If Not ActProcessActions(env) Then _
    Exit Function
  
  Build = True
  Call reporter.Log("Finish building product: " & Name)
End Function

Public Function Install(bType As TBuildType, reporter As API_Logger, dbComponents As DB_Components, sInstallPath$) As Boolean
  Install = False
  
  Call reporter.Log("Start installing product: " & Name)
  Call reporter.Log("Version: " & version_)
  
  Dim env As New ItemActionEnvironment
  Call env.InitInstall(bType, Name, version_, ArtifactHome, SourceHome, reporter, dbComponents, sInstallPath)
  
  Call reporter.Log("Validating makefile...")
  If Not ActValidateMakefile(env) Then _
    Exit Function

  Call reporter.Log("Processing installation actions...")
  If Not ActProcessActions(env) Then _
    Exit Function
    
  If distribute_ Then
    Call reporter.Log("Updating distribution manifest...")
    If Not UpdateDistribution Then
      Call reporter.Log("Failed to save distribution manifest...")
      Exit Function
    End If
  End If

  Install = True
  Call reporter.Log("Finish installing product: " & Name)
End Function

' =============
Private Function UpdateDistribution() As Boolean
  Dim sManifest$: sManifest = Globals.DistributionHome & "\" & FILE_DISTRIBUTION_MANIFEST
  Dim iDistribution As New API_DistrManifest
  Call iDistribution.LoadFrom(sManifest)
  Call iDistribution.SetVersion(make_.ProductID, version_)
  UpdateDistribution = iDistribution.SaveTo(sManifest)
End Function

Private Function ActValidateMakefile(env As ItemActionEnvironment) As Boolean
  ActValidateMakefile = False
  
  Dim oErrors As Collection: Set oErrors = make_.Validate(env.sourceHome_, env.artifactHome_, env.isBuilding_)
  If oErrors Is Nothing Then
    Call env.reporter_.Log("Validation failed")
    Exit Function
  ElseIf oErrors.Count > 0 Then
    Call env.reporter_.Log("Validation failed")
    Exit Function
  End If
  
  ActValidateMakefile = True
End Function

Private Function ActProcessActions(env As ItemActionEnvironment) As Boolean
  ActProcessActions = False
  
  Dim nAct&: nAct = 1
  Dim anAction As InfoAction
  Dim acts As Collection: Set acts = IIf(env.isBuilding_, make_.buildActs_, make_.installActs_)
  For Each anAction In acts
    Call env.reporter_.Log("A" & nAct & "/" & acts.Count & ": " & anAction.Description)
    On Error GoTo FAILED_ACTION
    If Not anAction.Execute(env) Then
FAILED_ACTION:
      Call env.reporter_.Log("Action failed: " & nAct)
      Exit Function
    End If
    On Error GoTo 0
    nAct = nAct + 1
  Next anAction
  
  ActProcessActions = True
End Function
