VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "IteratorProduct"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' _P_ - projects
Private Enum ProductStruct
  [_First] = 1
  
  S_P_ID = 1
  S_P_VERSION = 2
  S_P_NAME = 3
  S_P_DESCRIPTION = 4
  S_P_ARTIFACT_HOME = 5
  S_P_SOURCE_HOME = 6
  S_P_INSTALL_HOME = 7
  
  S_P_STATUS = 8
  S_P_LAST_BUILD = 9
  S_P_DISTRIBUTE = 10
  S_P_LAST_INSTALL = 11
  
  [_Last] = 11
End Enum

Public row_ As Long

Private data_ As Excel.Worksheet

Public Sub Init(target As Excel.Worksheet, Optional tRow& = FIRST_ROW)
  Set data_ = target
  row_ = tRow
End Sub

Public Function Increment(Optional inc& = 1)
  If row_ + inc > 0 Then _
    row_ = row_ + inc
End Function

Public Function GoFirst()
  row_ = FIRST_ROW
End Function

Public Function GoLast()
  row_ = data_.Columns(S_P_NAME).Find(vbNullString, LookAt:=xlWhole).Row - 1
End Function

Public Function IsDone() As Boolean
  IsDone = data_.Cells(row_, S_P_NAME) = vbNullString
End Function

Public Function RemoveRow()
  Call data_.Rows(row_).Delete
End Function

Public Function Model(dbComponents As DB_Components) As InfoProduct
  Dim oRes As New InfoProduct
  If Not oRes.Init(Makefile, Version, IsDistributed) Then _
    Exit Function
  
  Call SyncMakefile(oRes.make_)
  Call SyncManifests(oRes.make_, dbComponents)
  Set Model = oRes
End Function

Public Function SyncMakefile(iSource As InfoMakefile)
  ProductID = iSource.properties_(MAKEFILE_PROPS_ID)
  Name = iSource.properties_(MAKEFILE_PROPS_NAME)
  ArtifactHome = iSource.properties_(MAKEFILE_PROPS_ARTIFACT)
  SourceHome = iSource.properties_(MAKEFILE_PROPS_SOURCE)
  InstallHome = iSource.properties_(MAKEFILE_PROPS_INSTALL)
  If iSource.properties_.Exists(MAKEFILE_PROPS_DESCRIPTION) Then _
    Description = iSource.properties_(MAKEFILE_PROPS_DESCRIPTION)
  Call UpdateHyperlinks
End Function

Public Function SyncManifests(iSource As InfoMakefile, dbComponents As DB_Components)
  Dim anAction As InfoAction
  Dim sSource$: sSource = SourcePath
  Dim sProduct$: sProduct = iSource.properties_(MAKEFILE_PROPS_NAME)
  For Each anAction In iSource.buildActs_
    If Not anAction.type_ = T_ACT_BUILD Then _
      GoTo NEXT_ACTION
    
    Dim sManifest$: sManifest = CPath(anAction.args_(1)).ToGlobal(sSource).Text
    Dim iManifest As New InfoManifest
    If Not iManifest.Init(sManifest) Then _
      GoTo NEXT_ACTION
    
    Dim sComponent$: sComponent = iManifest.properties_(MANIFEST_PROPS_NAME)
    Dim iComp As IteratorComponent
    If dbComponents.Contains(sProduct, sComponent) Then
      Set iComp = dbComponents.Access(sProduct, sComponent)
    Else
      Set iComp = dbComponents.INew
    End If
    Call iComp.SyncManifest(sProduct, iManifest)

NEXT_ACTION:
  Next anAction
End Function

Public Function SetBuildStatus(newVal As TBuildStatus)
  Status = newVal
  BuildDate = Now
End Function

Public Function UpdateHyperlinks()
  Call XLUpdateHyperlink(data_.Cells(row_, S_P_INSTALL_HOME), InstallHome)
  Call XLUpdateHyperlink(data_.Cells(row_, S_P_ARTIFACT_HOME), ArtifactPath)
  Call XLUpdateHyperlink(data_.Cells(row_, S_P_SOURCE_HOME), SourcePath)
End Function

'===== Propertiy Get =====
Public Property Get Makefile() As String
  Makefile = Globals.SourceHome & "\" & SourceHome & "\" & MAKEFILE_NAME
End Property

Public Property Get SourcePath() As String
  SourcePath = CPath(SourceHome).ToGlobal(Globals.SourceHome).Text
End Property

Public Property Get ArtifactPath() As String
  ArtifactPath = CPath(ArtifactHome).ToGlobal(Globals.ArtifactHome).Text
End Property

Public Property Get ProductID() As String
  ProductID = data_.Cells(row_, S_P_ID)
End Property

Public Property Get Version() As String
  Version = data_.Cells(row_, S_P_VERSION)
End Property

Public Property Get Name() As String
  Name = data_.Cells(row_, S_P_NAME)
End Property

Public Property Get Description() As String
  Description = data_.Cells(row_, S_P_DESCRIPTION)
End Property

Public Property Get ArtifactHome() As String
  ArtifactHome = data_.Cells(row_, S_P_ARTIFACT_HOME)
End Property

Public Property Get SourceHome() As String
  SourceHome = data_.Cells(row_, S_P_SOURCE_HOME)
End Property

Public Property Get InstallHome() As String
  InstallHome = data_.Cells(row_, S_P_INSTALL_HOME)
End Property

Public Property Get Status() As TBuildStatus
  Status = data_.Cells(row_, S_P_STATUS)
End Property

Public Property Get BuildDate() As Double
  BuildDate = data_.Cells(row_, S_P_LAST_BUILD)
End Property

Public Property Get IsDistributed() As Boolean
  IsDistributed = data_.Cells(row_, S_P_DISTRIBUTE) = "1"
End Property

Public Property Get InstallDate() As Double
  InstallDate = data_.Cells(row_, S_P_LAST_INSTALL)
End Property

' ==== Property Let ====
Public Property Let ProductID(newVal$)
  data_.Cells(row_, S_P_ID) = newVal
End Property

Public Property Let Version(newVal$)
  data_.Cells(row_, S_P_VERSION) = newVal
End Property

Public Property Let Name(newVal$)
  data_.Cells(row_, S_P_NAME) = newVal
End Property

Public Property Let Description(newVal$)
  data_.Cells(row_, S_P_DESCRIPTION) = newVal
End Property

Public Property Let ArtifactHome(newVal$)
  data_.Cells(row_, S_P_ARTIFACT_HOME) = newVal
End Property

Public Property Let SourceHome(newVal$)
  data_.Cells(row_, S_P_SOURCE_HOME) = newVal
End Property

Public Property Let InstallHome(newVal$)
  data_.Cells(row_, S_P_INSTALL_HOME) = newVal
End Property

Public Property Let Status(newVal As TBuildStatus)
  data_.Cells(row_, S_P_STATUS) = newVal
End Property

Public Property Let BuildDate(newVal As Double)
  data_.Cells(row_, S_P_LAST_BUILD) = newVal
End Property

Public Property Let IsDistributed(newVal As Boolean)
  data_.Cells(row_, S_P_DISTRIBUTE) = IIf(newVal, "1", "0")
End Property

Public Property Let InstallDate(newVal As Double)
  data_.Cells(row_, S_P_LAST_INSTALL) = newVal
End Property
