VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "s_CompoundIntervals"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private intervals_ As CDS_CompoundIntervals

Public Function Setup()
  ' Mandatory setup function
    Set intervals_ = New CDS_CompoundIntervals
End Function

Public Function Teardown()
  ' Mandatory teardown function
End Function

Public Function t_IsEmpty()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Empty")
  Call Dev_ExpectEQ(0, intervals_.Count)
  Call Dev_ExpectTrue(intervals_.IsEmpty)
  
  Call Dev_NewCase("Not Empty")
  Call intervals_.AddItem(2, 2)
  Call Dev_ExpectEQ(1, intervals_.Count)
  Call Dev_ExpectFalse(intervals_.IsEmpty)

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_AddInverval()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Invalid interval")
  Call intervals_.AddItem(1, -1)
  Call Dev_ExpectEQ(0, intervals_.Count)
  
  Call Dev_NewCase("Zero length")
  Call intervals_.AddItem(2, 2)
  Call Dev_AssertEQ(1, intervals_.Count)
  Call Dev_ExpectEQ(Interval(2, 2), intervals_.items_(1))
  
  Call Dev_NewCase("Duplicate")
  Call intervals_.AddItem(2, 2)
  Call Dev_ExpectEQ(1, intervals_.Count)
  
  Call Dev_NewCase("Generic interval ordering")
  Call intervals_.AddItem(3, 4)
  Call intervals_.AddItem(-1, 0)
  Call Dev_AssertEQ(3, intervals_.Count)
  Call Dev_ExpectEQ(Interval(-1, 0), intervals_.items_(1))
  Call Dev_ExpectEQ(Interval(2, 2), intervals_.items_(2))
  Call Dev_ExpectEQ(Interval(3, 4), intervals_.items_(3))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_Compounding()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Absorb zero length")
  Call intervals_.AddItem(2, 2)
  Call intervals_.AddItem(1, 2)
  Call Dev_AssertEQ(1, intervals_.Count)
  Call Dev_ExpectEQ(Interval(1, 2), intervals_.items_(1))
  Call intervals_.AddItem(1, 1)
  Call Dev_ExpectEQ(1, intervals_.Count)
  
  Call Dev_NewCase("Merge borders")
  Call intervals_.Clear
  Call intervals_.AddItem(1, 2)
  Call intervals_.AddItem(3, 4)
  Call intervals_.AddItem(2, 3)
  Call Dev_AssertEQ(1, intervals_.Count)
  Call Dev_ExpectEQ(Interval(1, 4), intervals_.items_(1))
  
  Call Dev_NewCase("Absorb nested")
  Call intervals_.Clear
  Call intervals_.AddItem(1, 4)
  Call intervals_.AddItem(2, 3)
  Call Dev_AssertEQ(1, intervals_.Count)
  Call Dev_ExpectEQ(Interval(1, 4), intervals_.items_(1))
  
  Call Dev_NewCase("Merge intersecting")
  Call intervals_.Clear
  Call intervals_.AddItem(1, 3)
  Call intervals_.AddItem(2, 4)
  Call Dev_AssertEQ(1, intervals_.Count)
  Call Dev_ExpectEQ(Interval(1, 4), intervals_.items_(1))
  
  Call Dev_NewCase("Merge multiple")
  Call intervals_.Clear
  Call intervals_.AddItem(1, 3)
  Call intervals_.AddItem(4, 6)
  Call intervals_.AddItem(2, 5)
  Call Dev_AssertEQ(1, intervals_.Count)
  Call Dev_ExpectEQ(Interval(1, 6), intervals_.items_(1))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

' ======
Private Function Interval(nStart&, nFinish&) As CDS_Interval
  Dim iNewItem As New CDS_Interval: Call iNewItem.Init(nStart, nFinish)
  Set Interval = iNewItem
End Function
