VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "s_ExCollection"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' ===== Test suite for date Parsing =======
Option Explicit

Public Function Setup()
  ' Mandatory setup function
End Function

Public Function Teardown()
  ' Mandatory teardown function
End Function

Public Function t_InCollection()
  On Error GoTo PROPAGATE_ERROR
  
  Dim aCol As Collection
  Dim sKey$: sKey = "test"
  Call Dev_ExpectFalse(InCollection(sKey, Nothing), "Nothing")
  Call Dev_ExpectFalse(InCollection(sKey, aCol), "Undefined")
  
  Set aCol = New Collection
  Call Dev_ExpectFalse(InCollection(sKey, aCol), "Empty")
  
  Dim nItem&
  For nItem = 1 To 10 Step 1
    Call aCol.Add("", sKey & VBA.Int(VBA.Rnd * 1000))
  Next nItem
  Call Dev_ExpectFalse(InCollection(sKey, aCol), "False")
  
  Call aCol.Add("", sKey)
  Call Dev_ExpectTrue(InCollection(sKey, aCol), "True")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_IsArrayAllocated()
  On Error GoTo PROPAGATE_ERROR
  
  Dim tArray() As String
  
  Call Dev_ExpectFalse(IsArrayAllocated(0), "Non-array basic type")
  Call Dev_ExpectFalse(IsArrayAllocated(Nothing), "Nothing")
  Call Dev_ExpectFalse(IsArrayAllocated(Me), "Object")
  Call Dev_ExpectFalse(IsArrayAllocated(tArray), "No allocated")
  
  ReDim tArray(1 To 10)
  Call Dev_ExpectTrue(IsArrayAllocated(tArray), "Allocated")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_IsSubsetOf()
  On Error GoTo PROPAGATE_ERROR
  
  Dim aCol1 As New Collection
  Dim aCol2 As New Collection
  
  Call Dev_ExpectFalse(IsSubsetOf(Nothing, Nothing), "Nothing is not subset of nothing")
  Call Dev_ExpectFalse(IsSubsetOf(Nothing, aCol2), "Nothing is not subset of anything")
  Call Dev_ExpectFalse(IsSubsetOf(aCol1, Nothing), "Anything is not subset of Nothing")
  Call Dev_ExpectTrue(IsSubsetOf(aCol1, aCol2), "Empty is subset of Empty")
  
  Dim sKey1$: sKey1 = "test1"
  Dim sKey2$: sKey2 = "test2"
  Dim sKey3$: sKey3 = "test3"
  Call aCol2.Add(sKey1, sKey1)
  Call aCol2.Add(sKey2, sKey2)
  Call Dev_ExpectTrue(IsSubsetOf(aCol1, aCol2), "Empty is subset of Anything")
  
  Call aCol1.Add(sKey1, sKey1)
  Call Dev_ExpectTrue(IsSubsetOf(aCol1, aCol2), "Valid subset")
  
  Call aCol1.Add(sKey1, sKey2)
  Call Dev_ExpectTrue(IsSubsetOf(aCol1, aCol2), "Self subset")
  
  Call aCol1.Add(sKey3, sKey3)
  Call Dev_ExpectFalse(IsSubsetOf(aCol1, aCol2), "Not subset")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_SafeAddToCollection()
  On Error GoTo PROPAGATE_ERROR
  
  Dim aCol As Collection
  Dim sKey$: sKey = "test"
  Call Dev_ExpectFalse(SafeAddToCollection("", sKey, Nothing), "Nothing")
  Call Dev_ExpectFalse(SafeAddToCollection("", sKey, aCol), "Undefined")
  
  Call Dev_NewCase("Valid add")
  Set aCol = New Collection
  Call Dev_ExpectTrue(SafeAddToCollection("", sKey, aCol))
  Call Dev_ExpectTrue(InCollection(sKey, aCol))
  Call Dev_ExpectEQ(1, aCol.Count)
  
  Call Dev_NewCase("Add duplicate")
  Call Dev_ExpectFalse(SafeAddToCollection("", sKey, aCol))
  Call Dev_ExpectTrue(InCollection(sKey, aCol))
  Call Dev_ExpectEQ(1, aCol.Count)
  
  Call Dev_NewCase("Add second item")
  Dim sKey2$: sKey2 = "test2"
  Call Dev_ExpectTrue(SafeAddToCollection("", sKey2, aCol))
  Call Dev_ExpectTrue(InCollection(sKey2, aCol))
  Call Dev_ExpectEQ(2, aCol.Count)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_SafeMergeCollection()
  On Error GoTo PROPAGATE_ERROR
  
  Dim aCol1 As New Collection
  Dim aCol2 As New Collection
  
  Call Dev_ExpectFalse(SafeMergeCollection(Nothing, Nothing), "Nothing merge nothing")
  Call Dev_ExpectFalse(SafeMergeCollection(Nothing, aCol2), "Nothing merge anything")
  Call Dev_ExpectFalse(SafeMergeCollection(aCol1, Nothing), "Anything merge Nothing")
  Call Dev_ExpectTrue(SafeMergeCollection(aCol1, aCol2), "Empty merge Empty")
  Call Dev_ExpectTrue(SafeMergeCollection(aCol1, aCol1), "Merge self")
  
  Call Dev_NewCase("Valid merge")
  Dim sKey1$: sKey1 = "test1"
  Dim sKey2$: sKey2 = "test2"
  Dim sKey3$: sKey3 = "test3"
  Call aCol1.Add(sKey1, sKey1)
  Call aCol1.Add(sKey2, sKey2)
  Call aCol2.Add(sKey1, sKey1)
  Call aCol2.Add(sKey3, sKey3)
  Call Dev_ExpectTrue(SafeMergeCollection(aCol1, aCol2))
  Call Dev_ExpectEQ(2, aCol1.Count)
  Call Dev_ExpectEQ(3, aCol2.Count)
  Call Dev_ExpectTrue(InCollection(sKey2, aCol2))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_ArrayConversion()
  On Error GoTo PROPAGATE_ERROR
  
  Dim tArray As Variant
  Dim aCol As Collection
  
  Call Dev_ExpectNothing(FromArray(tArray), "Nothing from undef")
  Call Dev_ExpectFalse(IsArrayAllocated(ToArray(Nothing)), "Nothing to undef")
  
  Call Dev_NewCase("Empty to empty")
  Set aCol = New Collection
  tArray = ToArray(aCol)
  Call Dev_ExpectTrue(IsArrayAllocated(tArray))
  Call Dev_ExpectEQ(UBound(tArray, 1), LBound(tArray, 1))
  
  Call Dev_NewCase("Empty from empty")
  Set aCol = FromArray(tArray)
  Call Dev_AssertNotNothing(aCol)
  Call Dev_ExpectEQ(0, aCol.Count)
  
  Call Dev_NewCase("Valid collection to array")
  Set aCol = New Collection
  Const nCount& = 10
  Dim nItem&
  For nItem = 1 To nCount Step 1
    Dim nVal&: nVal = VBA.Int(VBA.Rnd * 1000)
    If Not InCollection(CStr(nVal), aCol) Then _
      Call aCol.Add(nVal, CStr(nVal))
  Next nItem
  tArray = ToArray(aCol)
  Call Dev_AssertEQ(nCount, UBound(tArray) - LBound(tArray) + 1)
  For nItem = 1 To nCount Step 1
    Call Dev_ExpectTrue(InCollection(CStr(tArray(nItem - 1)), aCol))
  Next nItem
  
  Call Dev_NewCase("Valid collection from array")
  Set aCol = Nothing
  Set aCol = FromArray(tArray)
  Call Dev_ExpectEQ(nCount, UBound(tArray) - LBound(tArray) + 1)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_CollectionToIndex()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectEQ(CSet(), CollectionToIndex(CColl()), "Empty collection")
  
  Call Dev_NewCase("Unique elements")
  Dim indicies As New Scripting.Dictionary
  indicies(1337) = 1
  indicies("leet") = 2
  Call Dev_ExpectEQ(indicies, CollectionToIndex(CColl(1337, "leet")))
  
  Call Dev_NewCase("Initialized duplicate elements")
  Call Dev_ExpectEQ(indicies, CollectionToIndex(CColl(1337, "leet", 1337)))

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_RevertCollection()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectEQ(CColl(), RevertCollection(CColl()), "Empty collection")
  
  Call Dev_NewCase("Unique elements")
  Dim indicies As New Scripting.Dictionary
  indicies(1337) = 1
  indicies("leet") = 2
  Call Dev_ExpectEQ(CColl("leet", 1337), RevertCollection(CColl(1337, "leet")))
  
  Call Dev_NewCase("Palindrom")
  Call Dev_ExpectEQ(CColl(1337, "leet", 1337), RevertCollection(CColl(1337, "leet", 1337)))

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function
