VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "s_Graph"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' ===== Test suite for Graph data structure =======
Option Explicit

Private graph_ As CDS_Graph

Public Function Setup()
  ' Mandatory setup function
  Set graph_ = New CDS_Graph
End Function

Public Function Teardown()
  ' Mandatory teardown function
End Function

Public Function t_AddNode()
  On Error GoTo PROPAGATE_ERROR
  
  Call graph_.AddNode(1337)
  Call Dev_ExpectEQ(1, graph_.Size, "Valid add")
  Call Dev_ExpectTrue(graph_.HasNode(1337))
  Call Dev_ExpectFalse(graph_.HasNode("1337"), "Do not use type conversion for node comparisons")
  
  Call graph_.AddNode(1337)
  Call Dev_ExpectEQ(1, graph_.Size, "Duplicate add")
  
  Call Dev_NewCase("Different node data types")
  Call graph_.AddNode("abc")
  Call Dev_ExpectEQ(2, graph_.Size)
  Call Dev_ExpectEQ(CColl(1337, "abc"), graph_.Nodes)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_AddEdge()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Valid edge")
  Call graph_.AddNode(1)
  Call graph_.AddNode(2)
  Dim iEdge As CDS_Edge: Set iEdge = graph_.AddEdge(1, 2)
  Call Dev_AssertNotNothing(iEdge)
  Call Dev_ExpectTrue(graph_.HasEdge(1, 2))
  Call Dev_ExpectEQ(1, iEdge.source_)
  Call Dev_ExpectEQ(2, iEdge.dest_)
  
  Call Dev_NewCase("Duplicate edge")
  Call Dev_ExpectNothing(graph_.AddEdge(1, 2))
  
  Call Dev_NewCase("Self edge")
  Set iEdge = graph_.AddEdge(1, 1)
  Call Dev_AssertNotNothing(iEdge)
  Call Dev_ExpectTrue(graph_.HasEdge(1, 1))
  Call Dev_ExpectEQ(1, iEdge.source_)
  Call Dev_ExpectEQ(1, iEdge.dest_)
  
  Call Dev_NewCase("Add missing nodes")
  Set iEdge = graph_.AddEdge(3, 4)
  Call Dev_AssertNotNothing(iEdge)
  Call Dev_ExpectTrue(graph_.HasEdge(3, 4))
  Call Dev_ExpectEQ(3, iEdge.source_)
  Call Dev_ExpectEQ(4, iEdge.dest_)
  Call Dev_ExpectEQ(4, graph_.Size, "Check node count")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_Clear()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Empty graph")
  On Error Resume Next
  Call graph_.Clear
  Call Dev_ExpectNoError
  On Error GoTo PROPAGATE_ERROR
  Call Dev_ExpectEQ(0, graph_.Size)
  
  Call Dev_NewCase("Valid graph")
  Call graph_.AddNode(1)
  Call graph_.AddEdge(2, 3)
  Call Dev_ExpectEQ(3, graph_.Size)
  Call Dev_ExpectTrue(graph_.HasEdge(2, 3))
  Call graph_.Clear
  Call Dev_ExpectEQ(0, graph_.Size)
  Call Dev_ExpectFalse(graph_.HasEdge(2, 3))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_FilterInternalEdges()
  On Error GoTo PROPAGATE_ERROR
  
  Dim iEdges As New Collection
  Call Dev_ExpectEQ(iEdges, graph_.FilterInternalEdges(CSet()), "Empty graph")
  
  Call Dev_NewCase("No edges")
  Call graph_.AddNode(1): Call graph_.AddNode(2)
  Call Dev_ExpectEQ(iEdges, graph_.FilterInternalEdges(CSet()), "Empty filter")
  Call Dev_ExpectEQ(iEdges, graph_.FilterInternalEdges(CSet(3, 4)), "Invalid nodes")
  Call Dev_ExpectEQ(iEdges, graph_.FilterInternalEdges(CSet(1, 2)), "No edges")
  
  Call Dev_NewCase("Valid filter")
  Dim edge1 As CDS_Edge: Set edge1 = graph_.AddEdge(1, 2)
  Dim edge2 As CDS_Edge: Set edge2 = graph_.AddEdge(1, 3)
  Call Dev_ExpectEQ(iEdges, graph_.FilterInternalEdges(CSet(2)), "Filter end")
  Call Dev_ExpectEQ(iEdges, graph_.FilterInternalEdges(CSet(1)), "Filter start")
  Call iEdges.Add(edge1.Clone)
  Call Dev_ExpectEQ(iEdges, graph_.FilterInternalEdges(CSet(1, 2)), "Filter internal")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_StrongComponents()
  On Error GoTo PROPAGATE_ERROR
  
  Dim iProcessor As New API_StrongComponents
  Dim iComponents As New Collection
  Call Dev_ExpectEQ(iComponents, iProcessor.GetComponents(graph_), "Empty graph")

  Call Dev_NewCase("No edges")
  Call graph_.AddNode(1): Call graph_.AddNode(2)
  Call Dev_ExpectEQ(iComponents, iProcessor.GetComponents(graph_))
  
  Call Dev_NewCase("Self edge")
  Set iComponents = CColl(CSet(1))
  Call graph_.AddEdge(1, 1)
  Call Dev_ExpectEQ(iComponents, iProcessor.GetComponents(graph_))
  
  Call Dev_NewCase("Valid strong components")
  Set iComponents = CColl(CSet(2, 3, 4), CSet(1))
  Call graph_.AddEdge(1, 2)
  Call graph_.AddEdge(2, 3)
  Call graph_.AddEdge(3, 2)
  Call graph_.AddEdge(2, 4)
  Call graph_.AddEdge(4, 2)
  Call Dev_ExpectEQ(iComponents, iProcessor.GetComponents(graph_))

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_LinkedComponents()
  On Error GoTo PROPAGATE_ERROR
  
  Dim iProcessor As New API_LinkedComponents
  
  Call Dev_NewCase("Empty graph")
  Dim iComponents As New Scripting.Dictionary
  Call Dev_ExpectEQ(iComponents, iProcessor.GetComponents(graph_))
  Call Dev_ExpectEQ(0, iProcessor.CountComponents)

  Call Dev_NewCase("No edges")
  Call graph_.AddNode(1): Call graph_.AddNode(2)
  iComponents(1) = 0
  iComponents(2) = 1
  Call Dev_ExpectEQ(iComponents, iProcessor.GetComponents(graph_))
  Call Dev_ExpectEQ(2, iProcessor.CountComponents)

  Call Dev_NewCase("Simple component - straight")
  Call graph_.AddEdge(1, 2)
  iComponents(1) = 0
  iComponents(2) = 0
  Call Dev_ExpectEQ(iComponents, iProcessor.GetComponents(graph_))
  Call Dev_ExpectEQ(1, iProcessor.CountComponents)
  
  Call Dev_NewCase("Simple component - reversed")
  Set graph_ = New CDS_Graph
  Call graph_.AddEdge(2, 1)
  iComponents(1) = 0
  iComponents(2) = 0
  Call Dev_ExpectEQ(iComponents, iProcessor.GetComponents(graph_))
  Call Dev_ExpectEQ(1, iProcessor.CountComponents)

  Call Dev_NewCase("Multiple components")
  Call graph_.AddEdge(3, 4)
  Call graph_.AddEdge(4, 3)
  Call graph_.AddEdge(4, 5)
  Call graph_.AddNode(6)
  iComponents(1) = 0
  iComponents(2) = 0
  iComponents(3) = 1
  iComponents(4) = 1
  iComponents(5) = 1
  iComponents(6) = 2
  Call Dev_ExpectEQ(iComponents, iProcessor.GetComponents(graph_))

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_Expansions()
  On Error GoTo PROPAGATE_ERROR
  
  Dim iEmpty As New Scripting.Dictionary
  Dim iTest As New Scripting.Dictionary
  
  Call Dev_NewCase("Empty graph")
  Call graph_.ExpandOutsOnce(iTest)
  Call Dev_ExpectEQ(iEmpty, iTest, "Outs once")
  Call graph_.ExpandInsOnce(iTest)
  Call Dev_ExpectEQ(iEmpty, iTest, "Ins once")
  Call graph_.ExpandBiderctionalOnce(iTest)
  Call Dev_ExpectEQ(iEmpty, iTest, "Both once")
  Call graph_.ExpandOutputs(iTest)
  Call Dev_ExpectEQ(iEmpty, iTest, "Outs all")
  Call graph_.ExpandInputs(iTest)
  Call Dev_ExpectEQ(iEmpty, iTest, "Ins all")
  Call graph_.ExpandBiderctional(iTest)
  Call Dev_ExpectEQ(iEmpty, iTest, "Both all")
  
  Call Dev_NewCase("Empty input")
  Call graph_.AddEdge(1, 2): Call graph_.AddEdge(3, 1)
  Call graph_.AddEdge(3, 4): Call graph_.AddEdge(5, 6): Call graph_.AddEdge(5, 8)
  Call graph_.AddEdge(6, 7): Call graph_.AddEdge(7, 6): Call graph_.AddEdge(9, 3)
  Call graph_.ExpandOutsOnce(iTest)
  Call Dev_ExpectEQ(iEmpty, iTest, "Outs once")
  Call graph_.ExpandInsOnce(iTest)
  Call Dev_ExpectEQ(iEmpty, iTest, "Ins once")
  Call graph_.ExpandBiderctionalOnce(iTest)
  Call Dev_ExpectEQ(iEmpty, iTest, "Both once")
  Call graph_.ExpandOutputs(iTest)
  Call Dev_ExpectEQ(iEmpty, iTest, "Outs all")
  Call graph_.ExpandInputs(iTest)
  Call Dev_ExpectEQ(iEmpty, iTest, "Ins all")
  Call graph_.ExpandBiderctional(iTest)
  Call Dev_ExpectEQ(iEmpty, iTest, "Both all")
  
  Call Dev_NewCase("Valid expansion")
  Set iTest = CSet(1, 5)
  Call graph_.ExpandOutsOnce(iTest)
  Call Dev_ExpectEQ(CSet(1, 5, 2, 8, 6), iTest, "Outs once")
  Set iTest = CSet(1, 5)
  Call graph_.ExpandInsOnce(iTest)
  Call Dev_ExpectEQ(CSet(1, 5, 3), iTest, "Ins once")
  Set iTest = CSet(1, 5)
  Call graph_.ExpandBiderctionalOnce(iTest)
  Call Dev_ExpectEQ(CSet(1, 5, 2, 8, 6, 3), iTest, "Both once")
  Set iTest = CSet(1, 5)
  Call graph_.ExpandOutputs(iTest)
  Call Dev_ExpectEQ(CSet(1, 5, 2, 8, 6, 7), iTest, "Outs all")
  Set iTest = CSet(1, 5)
  Call graph_.ExpandInputs(iTest)
  Call Dev_ExpectEQ(CSet(1, 5, 3, 9), iTest, "Ins all")
  Set iTest = CSet(1, 5)
  Call graph_.ExpandBiderctional(iTest)
  Call Dev_ExpectEQ(CSet(1, 5, 2, 3, 4, 6, 7, 8, 9), iTest, "Both all")

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_Ordering()
  On Error GoTo PROPAGATE_ERROR
  
  Dim iProcessor As New API_GraphOrdering: Call iProcessor.Init(graph_)
  Dim iEmpty As New Collection
  
  Call Dev_NewCase("Empty graph")
  Call Dev_ExpectEQ(iEmpty, iProcessor.TopologicalOrder)
  Call Dev_ExpectEQ(iEmpty, iProcessor.ReverseTopologicalOrder)
  Call Dev_ExpectEQ(iEmpty, iProcessor.TopologicalComponentOrder)
  Call Dev_ExpectEQ(iEmpty, iProcessor.ReverseTopologicalComponentOrder)
  
  Call Dev_NewCase("Empty input")
  Call graph_.AddEdge(1, 2): Call graph_.AddEdge(3, 1)
  Call graph_.AddEdge(3, 4): Call graph_.AddEdge(5, 6): Call graph_.AddEdge(5, 8)
  Call graph_.AddEdge(6, 7): Call graph_.AddEdge(7, 6): Call graph_.AddEdge(9, 3)
  Call iProcessor.Init(graph_)
  
  Call Dev_NewCase("Valid ordering")
  Call Dev_ExpectTrue(TestTopological(iProcessor.TopologicalOrder))
  Call Dev_ExpectTrue(TestTopological(RevertCollection(iProcessor.ReverseTopologicalOrder)))
  Call Dev_ExpectTrue(TestComponent(iProcessor.TopologicalComponentOrder))
  Call Dev_ExpectTrue(TestComponent(RevertCollection(iProcessor.ReverseTopologicalComponentOrder)))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_SortLayers()
  On Error GoTo PROPAGATE_ERROR
  
  Dim iProcessor As New API_GraphOrdering: Call iProcessor.Init(graph_)
  Dim iEmpty As New Collection
  
  Call Dev_ExpectEQ(iEmpty, iProcessor.SortLayers(iEmpty), "Empty graph")
  
  Call graph_.AddEdge(1, 2): Call graph_.AddEdge(3, 1)
  Call graph_.AddEdge(3, 4): Call graph_.AddEdge(5, 6): Call graph_.AddEdge(5, 8)
  Call graph_.AddEdge(6, 7): Call graph_.AddEdge(7, 6): Call graph_.AddEdge(9, 3)
  Call iProcessor.Init(graph_)
  
  Call Dev_ExpectEQ(iEmpty, iProcessor.SortLayers(iEmpty), "Empty input")
  Call Dev_ExpectEQ(iEmpty, iProcessor.SortLayers(CColl(1337, 42)), "Ignore invalid IDs")
  
  Call Dev_NewCase("Sorting")
  Call Dev_ExpectEQ(CColl(5, 6, 8, 7, 9, 3, 1, 4, 2), iProcessor.SortLayers(CColl(1, 2, 3, 4, 5, 6, 7, 8, 9)), "Full order")
  Call Dev_ExpectEQ(CColl(9, 3, 4, 1, 2, 5, 8, 6, 7), iProcessor.SortLayers(CColl(9, 8, 7, 6, 5, 4, 3, 2, 1)), "Reversed order")
  Call Dev_ExpectEQ(CColl(5, 6, 8, 7, 9, 3, 1, 4, 2), iProcessor.SortLayers(CColl(1, 2, 3, 4, 5, 6, 7, 8, 9, 1337, 42)), "Mixed invalid")
  Call Dev_ExpectEQ(CColl(5, 7, 9, 3, 1), iProcessor.SortLayers(CColl(1, 3, 5, 7, 9)), "Partial order")
  Call Dev_ExpectEQ(CColl(9, 1), iProcessor.SortLayers(CColl(1, 9)), "Missing component")
  
  Call Dev_NewCase("Sibling sorting")
  Call graph_.Clear
  Call graph_.AddEdge(1, 3): Call graph_.AddEdge(2, 3)
  Call graph_.AddEdge(4, 5): Call graph_.AddEdge(3, 5): Call graph_.AddEdge(3, 6)
  Call iProcessor.Init(graph_)
  Call Dev_ExpectEQ(CColl(1, 2, 4, 3, 5, 6), iProcessor.SortLayers(CColl(1, 2, 3, 4, 5, 6)))
  Call graph_.AddEdge(6, 5)
  Call iProcessor.Init(graph_)
  Call Dev_ExpectEQ(CColl(1, 2, 4, 3, 6, 5), iProcessor.SortLayers(CColl(1, 2, 3, 4, 5, 6)))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_SortDeep()
  On Error GoTo PROPAGATE_ERROR
  
  Dim iProcessor As New API_GraphOrdering: Call iProcessor.Init(graph_)
  Dim iEmpty As New Collection
  
  Call Dev_ExpectEQ(iEmpty, iProcessor.SortDeep(iEmpty), "Empty graph")
  
  Call graph_.AddEdge(1, 2): Call graph_.AddEdge(3, 1)
  Call graph_.AddEdge(3, 4): Call graph_.AddEdge(5, 6): Call graph_.AddEdge(5, 8)
  Call graph_.AddEdge(6, 7): Call graph_.AddEdge(7, 6): Call graph_.AddEdge(9, 3)
  Call iProcessor.Init(graph_)
  
  Call Dev_ExpectEQ(iEmpty, iProcessor.SortDeep(iEmpty), "Empty input")
  Call Dev_ExpectEQ(iEmpty, iProcessor.SortDeep(CColl(1337, 42)), "Ignore invalid IDs")
  
  Call Dev_NewCase("Sorting")
  Call Dev_ExpectEQ(CColl(5, 6, 7, 8, 9, 3, 1, 2, 4), iProcessor.SortDeep(CColl(1, 2, 3, 4, 5, 6, 7, 8, 9)), "Full order")
  Call Dev_ExpectEQ(CColl(9, 3, 4, 1, 2, 5, 8, 6, 7), iProcessor.SortDeep(CColl(9, 8, 7, 6, 5, 4, 3, 2, 1)), "Reversed order")
  Call Dev_ExpectEQ(CColl(5, 6, 7, 8, 9, 3, 1, 2, 4), iProcessor.SortDeep(CColl(1, 2, 3, 4, 5, 6, 7, 8, 9, 1337, 42)), "Mixed invalid")
  Call Dev_ExpectEQ(CColl(5, 7, 9, 3, 1), iProcessor.SortDeep(CColl(1, 3, 5, 7, 9)), "Partial order")
  Call Dev_ExpectEQ(CColl(9, 1), iProcessor.SortDeep(CColl(1, 9)), "Missing component")
  
  Call Dev_NewCase("Sibling sorting")
  Call graph_.Clear
  Call graph_.AddEdge(1, 3): Call graph_.AddEdge(2, 3)
  Call graph_.AddEdge(4, 5): Call graph_.AddEdge(3, 5): Call graph_.AddEdge(3, 6)
  Call iProcessor.Init(graph_)
  Call Dev_ExpectEQ(CColl(1, 3, 5, 6, 2, 4), iProcessor.SortDeep(CColl(1, 2, 3, 4, 5, 6)))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

' ======
Private Function TestTopological(iOrder As Collection) As Boolean
  TestTopological = False

  Dim nItem1&
  Dim nItem2&
  For nItem1 = iOrder.Count To 2 Step -1
    For nItem2 = 1 To nItem1 - 1 Step 1
      If graph_.HasEdge(iOrder(nItem1), iOrder(nItem2)) Then _
        If Not graph_.HasEdge(iOrder(nItem2), iOrder(nItem1)) Then _
          Exit Function
    Next nItem2
  Next nItem1
  
  TestTopological = True
End Function

Private Function TestComponent(iOrder As Collection) As Boolean
  TestComponent = False
  If Not TestTopological(iOrder) Then _
    Exit Function
  
  Dim iProcessor As New API_LinkedComponents
  Dim iComponents As Scripting.Dictionary: Set iComponents = iProcessor.GetComponents(graph_)
  
  Dim iVisited As New Scripting.Dictionary
  Dim nLast&: nLast = -1
  Dim nComponent&
  Dim vNode As Variant
  For Each vNode In iOrder
    nComponent = iComponents(vNode)
    If nLast <> nComponent Then
      If iVisited.Exists(nComponent) Then _
        Exit Function
      Call iVisited.Add(nComponent, 0)
      nLast = nComponent
    End If
  Next vNode
  
  TestComponent = True
End Function

