VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "s_JSON"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private json_ As API_JSON

Public Function Setup()
  ' Mandatory setup function
  Set json_ = New API_JSON
End Function

Public Function Teardown()
  ' Mandatory teardown function
End Function

Public Function t_GeneratePlain()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Invalid object")
  On Error Resume Next
  Call json_.CreateJSON(json_)
  Call Dev_ExpectAnyError
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Basic types")
  Call Dev_ExpectEQ("""""", json_.CreateJSON(""))
  Call Dev_ExpectEQ(json_.CreateJSON(0), "0")
  Call Dev_ExpectEQ(json_.CreateJSON(-1), "-1")
  Call Dev_ExpectEQ(json_.CreateJSON(1.234), "1.234")
  
  Call Dev_NewCase("String")
  Call Dev_ExpectEQ(json_.CreateJSON("test"), """test""")
  Call Dev_ExpectEQ(json_.CreateJSON(""), """""", "Cyrillic")
  Call Dev_ExpectEQ(json_.CreateJSON("1 "" 2"), """1 \"" 2""", "Escape quote")
  Call Dev_ExpectEQ(json_.CreateJSON("test1" & vbNewLine & "test2"), """test1\r\ntest2""", "Escape special symbol")
  
  Call Dev_NewCase("Array 1D")
  Call Dev_ExpectEQ(json_.CreateJSON(Array()), "[]")
  Call Dev_ExpectEQ(json_.CreateJSON(Array(1, 2, Array())), "[1,2,[]]")
  Call Dev_ExpectEQ(json_.CreateJSON(Array(Array(1, 2), 3)), "[[1,2],3]")
  
  Call Dev_NewCase("Array 2D")
  Dim testArr(1 To 2, 1 To 2) As Variant
  testArr(1, 1) = 1337
  testArr(2, 2) = 42
  Call Dev_ExpectEQ(json_.CreateJSON(testArr), "[[1337,null],[null,42]]")
  
  Call Dev_NewCase("Collection")
  Call Dev_ExpectEQ(json_.CreateJSON(CColl()), "[]")
  Call Dev_ExpectEQ(json_.CreateJSON(CColl(1, 2, 3)), "[1,2,3]")
  Call Dev_ExpectEQ(json_.CreateJSON(CColl(1, CColl(2, 3))), "[1,[2,3]]")
  
  Call Dev_NewCase("Dictionary")
  Dim testDict As New Scripting.Dictionary
  Call Dev_ExpectEQ(json_.CreateJSON(testDict), "{}")
  testDict.Item(1) = 42
  testDict.Item("test") = 1337
  Set testDict.Item(2) = CColl(1, 2)
  Call Dev_ExpectEQ(json_.CreateJSON(testDict), "{""1"":42,""test"":1337,""2"":[1,2]}")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_GenerateMods()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Escape unicode")
  json_.escapeUnicode_ = True
  Call Dev_ExpectEQ(json_.CreateJSON(""), """\u0442\u0435\u0441\u0442""")
  json_.escapeUnicode_ = False
  Call Dev_ExpectEQ(json_.CreateJSON(""), """""")
  
  Call Dev_NewCase("Escape slash")
  json_.escapeSlash_ = True
  Call Dev_ExpectEQ(json_.CreateJSON("/"), """\/""")
  json_.escapeSlash_ = False
  Call Dev_ExpectEQ(json_.CreateJSON("/"), """/""")
  
  Call Dev_NewCase("Pretty print")
  Dim test1$: test1 = _
    "[" & vbNewLine & _
    "  1," & vbNewLine & _
    "  [" & vbNewLine & _
    "    2," & vbNewLine & _
    "    3" & vbNewLine & _
    "  ]" & vbNewLine & _
    "]"
  Call json_.SetupMultiline(2)
  Call Dev_ExpectEQ(json_.CreateJSON(Array(1, Array(2, 3))), test1, "Print2")
  
  Dim test2$: test2 = _
    "[" & vbNewLine & _
    "    1," & vbNewLine & _
    "    [" & vbNewLine & _
    "        2," & vbNewLine & _
    "        3" & vbNewLine & _
    "    ]" & vbNewLine & _
    "]"
  Call json_.SetupMultiline(4)
  Call Dev_ExpectEQ(json_.CreateJSON(Array(1, Array(2, 3))), test2, "Print4")
  
  Dim test3$: test3 = _
    "[" & vbNewLine & _
    "-1," & vbNewLine & _
    "-[" & vbNewLine & _
    "--2," & vbNewLine & _
    "--3" & vbNewLine & _
    "-]" & vbNewLine & _
    "]"
  Call json_.SetupMultiline("-")
  Call Dev_ExpectEQ(json_.CreateJSON(Array(1, Array(2, 3))), test3, "Print-")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_ParseValid()
  On Error GoTo PROPAGATE_ERROR
  
  Dim testDict As Scripting.Dictionary
  
  Call Dev_NewCase("Empty")
  Call Dev_ExpectEQ(json_.Parse("[]"), CColl())
  Call Dev_ExpectEQ(json_.Parse("{}"), CSet())
  
  Call Dev_NewCase("Basic types")
  Call Dev_ExpectEQ(json_.Parse("[1,2,""3""]"), CColl(1, 2, "3"))
  Call Dev_ExpectEQ(json_.Parse("[1,2,null]"), CColl(1, 2, Null))
  Set testDict = New Scripting.Dictionary
  testDict.Item("key1") = 1.234
  testDict.Item("key2") = 0
  Call Dev_ExpectEQ(json_.Parse("{""key1"":1.234,""key2"":0}"), testDict)
  
  Call Dev_NewCase("Compound structures")
  Call Dev_ExpectEQ(json_.Parse("[1,[2,3],{}]"), CColl(1, CColl(2, 3), CSet()))
  Set testDict = New Scripting.Dictionary
  Set testDict.Item("key1") = CColl(1, 2)
  Set testDict.Item("key2") = CColl(42)
  Call Dev_ExpectEQ(json_.Parse("{""key1"":[1,2],""key2"":[42]}"), testDict)
  
  Call Dev_NewCase("Slashes")

  
  Call Dev_NewCase("Pretty printed")
  Call json_.SetupMultiline(2)
  Dim sInput$: sInput = json_.CreateJSON(Array(1, Array(2, 3)))
  Call Dev_ExpectEQ(json_.Parse(sInput), CColl(1, CColl(2, 3)))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_ParsePath()
  On Error GoTo PROPAGATE_ERROR
  
  Dim testDict As Scripting.Dictionary
  
  json_.escapeSlash_ = False
  Set testDict = New Scripting.Dictionary
  testDict.Item("key1") = "C:ToolsPython39-venvScriptspython.exe"
  Call Dev_ExpectEQ(json_.Parse("{""key1"": ""C:\Tools\Python39-venv\Scripts\python.exe""}"), testDict)
  testDict.Item("key1") = "C:\Tools\Python39-venv\Scripts\python.exe"
  Call Dev_ExpectEQ(json_.Parse("{""key1"": ""C:\\Tools\\Python39-venv\\Scripts\\python.exe""}"), testDict)
  testDict.Item("key1") = "C:/Tools/Python39-venv/Scripts/python.exe"
  Call Dev_ExpectEQ(json_.Parse("{""key1"": ""C:/Tools/Python39-venv/Scripts/python.exe""}"), testDict)
  testDict.Item("key1") = "C:/Tools/Python39-venv/Scripts/python.exe"
  Call Dev_ExpectEQ(json_.Parse("{""key1"": ""C:\/Tools\/Python39-venv\/Scripts\/python.exe""}"), testDict)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_ParseInvalid()
  On Error GoTo PROPAGATE_ERROR
  
  Call ExpectParserError("")
  Call ExpectParserError("]")
  Call ExpectParserError("}")
  Call ExpectParserError("{")
  Call ExpectParserError("[")
  Call ExpectParserError("[[]")
  Call ExpectParserError("[]]")
  Call ExpectParserError("{{}")
  Call ExpectParserError("{}}")
  Call ExpectParserError("abc")
  Call ExpectParserError("{key:1}")
  Call ExpectParserError("[{]}")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

' ======
Private Function ExpectParserError(sInput$, Optional sClause$ = "")
   On Error Resume Next
  Call json_.Parse(sInput)
  Call Dev_ExpectAnyError(sClause & vbNewLine & "Input: " & sInput)
  On Error GoTo 0
End Function
