VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "s_Logger"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private fso_ As Scripting.FileSystemObject
Private logFile_ As String
Private log_ As API_Logger

Private Const TEST_LOG_NAME = "testlog.log"

Public Function Setup()
  ' Mandatory setup function
  Set fso_ = New Scripting.FileSystemObject
  Call Teardown
  
  logFile_ = ThisWorkbook.Path & "\" & TEST_LOG_NAME
  Set log_ = New API_Logger
  Call log_.Init(logFile_)
End Function

Public Function Teardown()
  ' Mandatory teardown function
  If fso_.FileExists(logFile_) Then _
    Call fso_.DeleteFile(logFile_)
End Function

Public Function t_Init()
  On Error GoTo PROPAGATE_ERROR
  
  Dim newLog As New API_Logger
  
  Call Dev_NewCase("Empty Init")
  On Error Resume Next
  Call newLog.Init("")
  Call Dev_ExpectNoError
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("File Init")
  Dim sFile1$: sFile1 = logFile_ & "1"
  Call newLog.Init(sFile1)
  
  Call Dev_ExpectTrue(fso_.FileExists(logFile_))
  Call Dev_ExpectEQ(sFile1, newLog.LogFileName)
  Dim cLines As Collection: Set cLines = ReadLogLines(sFile1)
  Call Dev_AssertEQ(1, cLines.Count)
  Call Dev_ExpectLike(cLines(1), "*Log file created")
  
  Call Dev_NewCase("Reinit")
  Dim sFile2$: sFile2 = logFile_ & "2"
  Call newLog.Init(sFile2)
  Call Dev_ExpectTrue(fso_.FileExists(logFile_))
  Call Dev_ExpectTrue(fso_.FileExists(sFile2))
  
  Call fso_.DeleteFile(sFile1)
  Call fso_.DeleteFile(sFile2)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_Timestamp()
  On Error GoTo PROPAGATE_ERROR
  
  Dim cLines As Collection: Set cLines = ReadLogLines(logFile_)
  Call Dev_AssertEQ(1, cLines.Count)
  Dim dTime As Double: dTime = ExtractTimestamp(cLines(1))
  Call Dev_ExpectAEQ(Now(), dTime, 0, "Valid stamp")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_Log()
  On Error GoTo PROPAGATE_ERROR

  Call Dev_NewCase("Valid messaging")
  Call log_.Log("")
  Call log_.Log("test")
  Call log_.Log("Multiline: line1" & vbNewLine & "Multiline: line2")
  Dim cLines As Collection: Set cLines = ReadLogLines(logFile_)
  Call Dev_AssertEQ(5, cLines.Count)
  Call Dev_ExpectEQ("Log file created", TrimTimestamp(cLines(1)))
  Call Dev_ExpectEQ("", TrimTimestamp(cLines(2)))
  Call Dev_ExpectEQ("test", TrimTimestamp(cLines(3)))
  Call Dev_ExpectEQ("Multiline: line1", TrimTimestamp(cLines(4)))
  Call Dev_ExpectEQ("Multiline: line2", TrimTimestamp(cLines(5)))

  Call Dev_NewCase("Try logging when log was deleted")
  Call fso_.DeleteFile(logFile_)
  On Error Resume Next
  Call log_.Log("test")
  Call Dev_ExpectNoError
  Call Dev_ExpectFalse(fso_.FileExists(logFile_))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

' =====
Private Function ReadLogLines(sFile$) As Collection
  Const UTF16_MODE = TristateTrue
  Set ReadLogLines = New Collection
  Dim fso As New Scripting.FileSystemObject
  Dim fs As Scripting.TextStream: Set fs = fso.OpenTextFile(sFile, ForReading, Format:=UTF16_MODE)
  Do While Not fs.AtEndOfStream
    Call ReadLogLines.Add(fs.ReadLine)
  Loop
  Call fs.Close
End Function
