VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "s_ParseDate"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' ===== Test suite for date Parsing =======
Option Explicit

Private parser_ As ParserDate

Public Function Setup()
  ' Mandatory setup function
  Set parser_ = New ParserDate
End Function

Public Function Teardown()
  ' Mandatory teardown function
End Function

Public Function t_DefaultState()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectEQ(0, parser_.day_)
  Call Dev_ExpectEQ(0, parser_.month_)
  Call Dev_ExpectEQ(0, parser_.year_)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_Init()
  On Error GoTo PROPAGATE_ERROR
  
  Call parser_.Init(12, 1, 20)
  Call Dev_NewCase("Init real date")
  Call Dev_ExpectEQ(12, parser_.day_)
  Call Dev_ExpectEQ(1, parser_.month_)
  Call Dev_ExpectEQ(20, parser_.year_)
  
  Call Dev_NewCase("Init invalid date")
  Call parser_.Init(99, 99, 20)
  Call Dev_ExpectEQ(99, parser_.day_)
  Call Dev_ExpectEQ(99, parser_.month_)
  Call Dev_ExpectEQ(20, parser_.year_)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_Date()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Empty")
  Call Dev_ExpectFalse(parser_.IsValidDate)
  Call Dev_ExpectEQ(0, parser_.DDate)
  
  Call Dev_NewCase("Valid date value")
  Call parser_.Init(31, 1, 20)
  Call Dev_ExpectTrue(parser_.IsValidDate)
  Call Dev_ExpectEQ(43861#, parser_.DDate)
  
  Call Dev_NewCase("Invalid date value")
  Call parser_.Init(32, 1, 20)
  Call Dev_ExpectFalse(parser_.IsValidDate)
  Call Dev_ExpectEQ(0#, parser_.DDate)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_AsString()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Empty")
  Call Dev_ExpectEQ("00.00.00", parser_.AsDigitsString)
  Call Dev_ExpectEQ("INVALID_DATE", parser_.AsTextString)
  
  Call Dev_NewCase("Short year")
  Call parser_.Init(31, 1, 20)
  Call Dev_ExpectEQ("31.01.20", parser_.AsDigitsString)
  Call Dev_ExpectEQ("31  2020 ", parser_.AsTextString)
  
  Call Dev_NewCase("Long year")
  Call parser_.Init(31, 1, 1989)
  Call Dev_ExpectEQ("31.01.1989", parser_.AsDigitsString)
  Call Dev_ExpectEQ("31  1989 ", parser_.AsTextString)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_Test()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Empty string")
  Call Dev_ExpectFalse(parser_.Test(""))
  
  Call Dev_NewCase("Invalid inputs")
  Call Dev_ExpectFalse(parser_.Test("123"))
  Call Dev_ExpectFalse(parser_.Test("12  2000 "))
  
  Call Dev_NewCase("Incomplete inputs")
  Call Dev_ExpectFalse(parser_.Test("12  20"))
  Call Dev_ExpectFalse(parser_.Test(" 2000 "))
  Call Dev_ExpectFalse(parser_.Test("13 2000 "))
  Call Dev_ExpectFalse(parser_.Test("13  00 "))
  
  Call Dev_ExpectTrue(parser_.Test("13.01.2000"), "Valid input: 13.01.2000")
  Call Dev_ExpectTrue(parser_.Test("13.01.2000 ."), "Valid input: 13.01.2000 .")
  Call Dev_ExpectTrue(parser_.Test("13.01.00"), "Valid input: 13.01.00")
  Call Dev_ExpectTrue(parser_.Test("13  2000 "), "Valid input: 13  2000 ")
  Call Dev_ExpectTrue(parser_.Test("13  2000 "), "Valid input: 13  2000 ")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_Parse()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectFalse(parser_.Parse(""), "Parse empty string")
  Call Dev_ExpectFalse(parser_.Parse("123"), "Parse invalid")
  
  Call Dev_NewCase("Parse 13.01.2000")
  Call Dev_AssertTrue(parser_.Parse("13.01.2000"))
  Call Dev_ExpectEQ(13, parser_.day_)
  Call Dev_ExpectEQ(1, parser_.month_)
  Call Dev_ExpectEQ(2000, parser_.year_)
  
  Call Dev_NewCase("Do not reset after fail?")
  Call Dev_AssertFalse(parser_.Parse("123"))
  Call Dev_ExpectEQ(13, parser_.day_)
  Call Dev_ExpectEQ(1, parser_.month_)
  Call Dev_ExpectEQ(2000, parser_.year_)
  
  Call Dev_NewCase("Parse 13  2000 ")
  Call Dev_AssertTrue(parser_.Parse("13  2000 "))
  Call Dev_ExpectEQ(13, parser_.day_)
  Call Dev_ExpectEQ(1, parser_.month_)
  Call Dev_ExpectEQ(2000, parser_.year_)
  
  Call Dev_NewCase("Parse 13.01.00")
  Call Dev_AssertTrue(parser_.Parse("13.01.00"))
  Call Dev_ExpectEQ(13, parser_.day_)
  Call Dev_ExpectEQ(1, parser_.month_)
  Call Dev_ExpectEQ(0, parser_.year_)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function
