VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "s_StaticHierarchy"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private iTree_ As CDS_StaticHierarchy

Public Function Setup()
  ' Mandatory setup function
  Set iTree_ = New CDS_StaticHierarchy
End Function

Public Function Teardown()
  ' Mandatory teardown function
End Function

Public Function t_SingleRoot()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Empty")
  Call Dev_ExpectEQ(0, iTree_.Size, "Count nodes")
  Call Dev_ExpectEQ(0, iTree_.MaxDepth, "Max Depth")
  
  Call Dev_NewCase("Push root")
  Dim iRoot As CDS_NodeSH
  Set iRoot = iTree_.PushItem(1)
  Call Dev_AssertNotNothing(iRoot)
  Call Dev_ExpectEQ(1, iTree_.Size, "Add node")
  Call Dev_ExpectEQ(iRoot, iTree_.nodes_(1), "Add node")
  Call Dev_ExpectEQ(1, iTree_.MaxDepth, "Max Depth")
  Call Dev_ExpectEQ(1, iRoot.id_, "ID")
  Call Dev_ExpectEQ(1, iRoot.rank_, "Rank")
  Call Dev_ExpectNothing(iRoot.parent_, "Parent")
  Call Dev_ExpectEQ(0, iRoot.children_.Count, "Children")
  Call Dev_ExpectNothing(iRoot.data_, "Payload")
  
  Call Dev_NewCase("Push regular structure")
  Call Dev_ExpectNotNothing(iTree_.PushItem(2))
  Call Dev_ExpectNotNothing(iTree_.PushItem(3))
  Call Dev_ExpectNotNothing(iTree_.PushItem(4))
  Call Dev_ExpectNotNothing(iTree_.PushItem(2))
  Call Dev_ExpectEQ(5, iTree_.Size, "Count nodes")
  Call Dev_ExpectEQ(4, iTree_.MaxDepth, "Max Depth")
  Call Dev_ExpectEQ(2, iRoot.children_.Count, "Count children")
  Call Dev_ExpectEQ(iRoot, iRoot.children_(1).parent_, "Parent assignment")
  Call Dev_ExpectEQ(iRoot, iRoot.children_(2).parent_, "Parent assignment")

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_MultipleRoots()
  On Error GoTo PROPAGATE_ERROR

  Call iTree_.PushItem(2)
  Call iTree_.PushItem(3)
  
  Call Dev_NewCase("New root same rank")
  Call Dev_AssertNotNothing(iTree_.PushItem(2))
  Call Dev_ExpectNothing(iTree_.nodes_(3).parent_, "New root")
  Call Dev_ExpectNothing(iTree_.nodes_(1).parent_, "Old root")
  
  Call Dev_NewCase("New root lower rank")
  Call Dev_AssertNotNothing(iTree_.PushItem(1))
  Call Dev_AssertNotNothing(iTree_.PushItem(2))
  Call Dev_ExpectNothing(iTree_.nodes_(4).parent_, "New root")
  Call Dev_ExpectNothing(iTree_.nodes_(1).parent_, "Old root")
  Call Dev_ExpectEQ(1, iTree_.nodes_(4).children_.Count, "Do not assume old roots as children!")

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_StructuralErrors()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Invalid level gap")
  Call iTree_.PushItem(10)
  Call Dev_ExpectNothing(iTree_.PushItem(12))
  
  Call Dev_ExpectNotNothing(iTree_.PushItem(11))
  Call Dev_ExpectNotNothing(iTree_.PushItem(12))
  Call Dev_ExpectNotNothing(iTree_.PushItem(13))
  Call Dev_ExpectNotNothing(iTree_.PushItem(11))
  Call Dev_ExpectNothing(iTree_.PushItem(13))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_CountDescendants()
  On Error GoTo PROPAGATE_ERROR
  
  Call iTree_.PushItem(1) ' 1: 1
  Call iTree_.PushItem(2) ' 2: 1.1
  Call iTree_.PushItem(3) ' 3: 1.1.1
  Call iTree_.PushItem(3) ' 4: 1.1.2
  Call iTree_.PushItem(1) ' 5: 2
  Call iTree_.PushItem(1) ' 6: 3
  Call iTree_.PushItem(2) ' 7: 3.1
  Call iTree_.PushItem(2) ' 8: 3.2
  
  Call Dev_ExpectEQ(3, iTree_.nodes_(1).descendantsCount_, "Node 1")
  Call Dev_ExpectEQ(2, iTree_.nodes_(2).descendantsCount_, "Node 2")
  Call Dev_ExpectEQ(0, iTree_.nodes_(3).descendantsCount_, "Node 3")
  Call Dev_ExpectEQ(0, iTree_.nodes_(4).descendantsCount_, "Node 4")
  Call Dev_ExpectEQ(0, iTree_.nodes_(5).descendantsCount_, "Node 5")
  Call Dev_ExpectEQ(2, iTree_.nodes_(6).descendantsCount_, "Node 6")
  Call Dev_ExpectEQ(0, iTree_.nodes_(7).descendantsCount_, "Node 7")
  Call Dev_ExpectEQ(0, iTree_.nodes_(8).descendantsCount_, "Node 8")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function
