VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "s_TextEdit"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' ===== Test suite for date Parsing =======
Option Explicit

Private Const STYLE_TEST_PARA$ = "TestParagraph"
Private Const STYLE_TEST_CHAR$ = "TestCharacter"

Private doc_ As Word.Document

Public Function Setup()
  ' Mandatory setup function
  Set doc_ = ThisDocument
End Function

Public Function Teardown()
  ' Mandatory teardown function
  Call doc_.Range.Delete
  Call doc_.Range.Select
  Call Word.Selection.ClearFormatting
End Function

Public Function t_AdjustRange()
  On Error GoTo PROPAGATE_ERROR
  ' =========== 12345678900123456678
  doc_.Range = " """" 1" & vbNewLine & " "
  
  Call Dev_ExpectEQ("", WordAdjustRange(doc_.Range(0, 0)), "Start of document")
  Call Dev_ExpectEQ(" ", WordAdjustRange(doc_.Range(1, 1)), "Zero length")
  Call Dev_ExpectEQ(" ", WordAdjustRange(doc_.Range(1, 8)), "Valid word")
  Call Dev_ExpectEQ(" ", WordAdjustRange(doc_.Range(2, 7)), "Partial word")
  Call Dev_ExpectEQ(" ", WordAdjustRange(doc_.Range(1, 9)), "Ignore end space")
  Call Dev_ExpectEQ(" """" ", WordAdjustRange(doc_.Range(1, 11)), "Quoted words")
  Call Dev_ExpectEQ(""""" ", WordAdjustRange(doc_.Range(12, 13)), "Do not extend leading space")
  Call Dev_ExpectEQ(" ", WordAdjustRange(doc_.Range(8, 9)), "Single space")
  Call Dev_ExpectEQ(doc_.Paragraphs.First.Range, WordAdjustRange(doc_.Paragraphs.First.Range), "Whole paragraph")

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_AddText()
  On Error GoTo PROPAGATE_ERROR
  
  doc_.Range = "1" & vbNewLine & "2" & vbNewLine & "3"
  
  Call Dev_NewCase("Put text in position")
  Dim sTxt1$: sTxt1 = "testI"
  Dim firstPara As Word.Range: Set firstPara = doc_.Paragraphs(1).Range
  Dim insertedRes As Word.Range: Set insertedRes = WordPutText(sTxt1, doc_, 0)
  Call Dev_ExpectEQ(sTxt1, insertedRes.Text)
  Call Dev_ExpectEQ(Len(sTxt1) + 2, firstPara.End)
  
  Call Dev_NewCase("Add text after")
  Dim secondPara As Word.Range: Set secondPara = doc_.Paragraphs(2).Range
  Dim sTxt2$: sTxt2 = "testII"
  Set insertedRes = WordAddLine(sTxt2, secondPara, STYLE_TEST_PARA)
  Call insertedRes.MoveEnd(wdCharacter, -1)
  Call Dev_ExpectEQ(doc_.Paragraphs(2).Range.End, secondPara.End)
  Call Dev_ExpectEQ(secondPara.End, insertedRes.Start)
  Call Dev_ExpectEQ(sTxt2, insertedRes.Text)
  Call Dev_ExpectEQ(STYLE_TEST_PARA, insertedRes.Style)
  
  Call Dev_NewCase("Append text")
  Dim sTxt3$: sTxt3 = "testIII"
  Set secondPara = secondPara.Characters.First
  Set insertedRes = WordAppendTo(sTxt3, secondPara, STYLE_TEST_CHAR)
  Call Dev_ExpectEQ(doc_.Paragraphs(2).Range.Characters.First.End + Len(sTxt3), secondPara.End)
  Call Dev_ExpectEQ("2" & sTxt3, secondPara.Text)
  Call Dev_ExpectNE(STYLE_TEST_CHAR, secondPara.Characters.First)
  Call Dev_ExpectEQ(insertedRes.Text, sTxt3)
  Call Dev_ExpectEQ(STYLE_TEST_CHAR, insertedRes.Style)

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_AddTextToEnd()
  On Error GoTo PROPAGATE_ERROR
  
  Call WordAddLine("test1", doc_.Content)
  Call Dev_ExpectEQ(2, doc_.Content.Paragraphs.Count)
  
  Call WordAppendTo("test2", doc_.Content)
  Call Dev_ExpectEQ(2, doc_.Content.Paragraphs.Count, "Append to empty line at document end")
  
  Call WordAppendTo("test3", doc_.Content)
  Call Dev_ExpectEQ(2, doc_.Content.Paragraphs.Count, "Append to non-empty line at document end")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_AddTextToTable()
  On Error GoTo PROPAGATE_ERROR
  Dim sTxt$: sTxt = "test"
  
  Dim aTable As Word.Table: Set aTable = doc_.Tables.Add(doc_.Content, 1, 2)
  aTable.Cell(1, 1).Range = "123"
  Dim newRng As Word.Range: Set newRng = WordAddLine(sTxt, aTable.Cell(1, 1).Range, STYLE_TEST_PARA)
  Dim cellRng As Word.Range: Set cellRng = aTable.Cell(1, 1).Range
  Call Dev_AssertEQ(2, cellRng.Paragraphs.Count)
  Call Dev_ExpectEQ(sTxt, newRng.Text, "Add text after")
  Call Dev_ExpectEQ(STYLE_TEST_PARA, newRng.Style, "Add text after")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_StyleExists()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectFalse(WordStyleExists(doc_, ""), "Empty input")
  Call Dev_ExpectFalse(WordStyleExists(doc_, "Invalid1234567"), "Invalid style")
  Call Dev_ExpectTrue(WordStyleExists(doc_, doc_.Paragraphs.First.Range.Style), "Default style")
  Call Dev_ExpectTrue(WordStyleExists(doc_, doc_.Paragraphs.First.Range.Style.NameLocal), "Default style local")
  Call Dev_ExpectTrue(WordStyleExists(doc_, STYLE_TEST_PARA), "Paragraph style")
  Call Dev_ExpectTrue(WordStyleExists(doc_, STYLE_TEST_CHAR), "Character style")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function
