VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "s_UndoWrapper"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' ===== Test suite for UndoWrapper =======
Option Explicit

Private wrapper_ As API_UndoWrapper
Private targetPage_ As Visio.Page

Public Function Setup()
  ' Mandatory setup function
  Set wrapper_ = New API_UndoWrapper
  Call wrapper_.Init(ThisDocument.Application)
  Set targetPage_ = ThisDocument.Pages(1)
End Function

Public Function Teardown()
  ' Mandatory teardown function
  Call ThisDocument.Application.PurgeUndo
End Function

Public Function t_BasicUndo()
  On Error GoTo PROPAGATE_ERROR
  
  Dim oldCount&: oldCount = targetPage_.Shapes.Count
  
  Call wrapper_.BeginScope("Test")
  Call AddTwoRectangles
  Call wrapper_.EndScope
  
  Call targetPage_.Application.Undo
  Call Dev_ExpectEQ(oldCount, targetPage_.Shapes.Count)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_CancelOnEndScope()
  On Error GoTo PROPAGATE_ERROR
  
  Dim oldCount&: oldCount = targetPage_.Shapes.Count
  
  Call wrapper_.BeginScope("Test")
  Call AddTwoRectangles
  Call wrapper_.EndScope(bCommit:=False)
  
  Call Dev_ExpectEQ(oldCount, targetPage_.Shapes.Count)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_EndBeforeStart()
  On Error GoTo PROPAGATE_ERROR
  
  Call wrapper_.EndScope
  Call wrapper_.EndScope(bCommit:=False)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_DoubleStart()
  On Error GoTo PROPAGATE_ERROR
  
  Dim oldCount&: oldCount = targetPage_.Shapes.Count
  
  Call wrapper_.BeginScope("Test")
  Call wrapper_.BeginScope("Test")
  Call AddTwoRectangles
  Call wrapper_.BeginScope("Test2")
  Call AddTwoRectangles
  Call wrapper_.EndScope
  
  Call targetPage_.Application.Undo
  Call Dev_ExpectEQ(oldCount, targetPage_.Shapes.Count, "Nested scope doesnt work. Only first scope is relevant")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

' ======
Private Function AddTwoRectangles()
  Dim oldCount&: oldCount = targetPage_.Shapes.Count
  Call targetPage_.DrawRectangle(1, 1, 2, 2)
  Call targetPage_.DrawRectangle(3, 3, 4, 4)
  Call Dev_AssertEQ(oldCount + 2, targetPage_.Shapes.Count)
End Function
